@@ -34,6 +34,7 @@
 
 #include <pbd/error.h>
 #include <pbd/compose.h>
+#include <pbd/misc.h>
 #include <pbd/pathscanner.h>
 #include <pbd/failed_constructor.h>
 #include <pbd/enumwriter.h>
@@ -97,7 +98,7 @@ sigc::signal<void,nframes_t, bool, nframes_t> ARDOUR_UI::Clock;
 ARDOUR_UI::ARDOUR_UI (int *argcp, char **argvp[])
 
 	: Gtkmm2ext::UI (X_("Ardour"), argcp, argvp),
-	  
+
 	  primary_clock (X_("primary"), false, X_("TransportClockDisplay"), true, false, true),
 	  secondary_clock (X_("secondary"), false, X_("SecondaryClockDisplay"), true, false, true),
 	  preroll_clock (X_("preroll"), false, X_("PreRollClock"), true, true),
@@ -148,11 +149,20 @@ ARDOUR_UI::ARDOUR_UI (int *argcp, char **argvp[])
 
 	  auditioning_alert_button (_("AUDITION")),
 	  solo_alert_button (_("SOLO")),
-	  shown_flag (false)
+	  shown_flag (false),
+	  error_log_button (_("Errors"))
 {
 	using namespace Gtk::Menu_Helpers;
 
 	Gtkmm2ext::init();
+
+#ifdef TOP_MENUBAR
+	_auto_display_errors = false;
+#endif
+
+	if (getenv ("ARDOUR_DEBUG_UPDATES")) {
+		gdk_window_set_debug_updates (true);
+	}
 	
 	about = 0;
 
@@ -163,12 +173,14 @@ ARDOUR_UI::ARDOUR_UI (int *argcp, char **argvp[])
 	ui_config = new UIConfiguration();
 	theme_manager = new ThemeManager();
 
+	engine = 0;
 	editor = 0;
 	mixer = 0;
 	session = 0;
 	_session_is_new = false;
 	big_clock_window = 0;
 	session_selector_window = 0;
+	new_session_dialog = 0;
 	last_key_press_time = 0;
 	connection_editor = 0;
 	add_route_dialog = 0;
@@ -180,6 +192,7 @@ ARDOUR_UI::ARDOUR_UI (int *argcp, char **argvp[])
 	have_disk_speed_dialog_displayed = false;
 	_will_create_new_session_automatically = false;
 	session_loaded = false;
+	loading_dialog = 0;
 	last_speed_displayed = -1.0f;
 
 	keybindings_path = ARDOUR::find_config_file ("ardour.bindings");
@@ -210,31 +223,81 @@ ARDOUR_UI::ARDOUR_UI (int *argcp, char **argvp[])
 
 	ARDOUR::Session::AskAboutPendingState.connect (mem_fun(*this, &ARDOUR_UI::pending_state_dialog));
 
-	/* have to wait for AudioEngine and Configuration before proceeding */
+	/* lets get this party started */
+
+    	try {
+		ARDOUR::init (ARDOUR_COMMAND_LINE::use_vst, ARDOUR_COMMAND_LINE::try_hw_optimization);
+		setup_gtk_ardour_enums ();
+		Config->set_current_owner (ConfigVariableBase::Interface);
+		setup_profile ();
+
+	} catch (failed_constructor& err) {
+		error << _("could not initialize Ardour.") << endmsg;
+		// pass it on up
+		throw err;
+	} 
+
+	/* we like keyboards */
+
+	keyboard = new Keyboard;
+
+	starting.connect (mem_fun(*this, &ARDOUR_UI::startup));
+	stopping.connect (mem_fun(*this, &ARDOUR_UI::shutdown));
+}
+
+gint
+ARDOUR_UI::start_backend_audio ()
+{
+	if (new_session_dialog->engine_control.start_engine ()) {
+		return -1;
+	} 
+
+	return 0;
 }
 
 void
-ARDOUR_UI::set_engine (AudioEngine& e)
+ARDOUR_UI::create_engine ()
 {
-	engine = &e;
+	// this gets called every time by new_session()
+
+	if (engine) {
+		return;
+	}
+
+	try { 
+		engine = new ARDOUR::AudioEngine (ARDOUR_COMMAND_LINE::jack_client_name);
+	} catch (AudioEngine::NoBackendAvailable& err) {
+		backend_audio_error ();
+		error << string_compose (_("Could not connect to JACK server as  \"%1\""), ARDOUR_COMMAND_LINE::jack_client_name) <<  endmsg;
+		exit (1);
+	}
 
 	engine->Stopped.connect (mem_fun(*this, &ARDOUR_UI::engine_stopped));
 	engine->Running.connect (mem_fun(*this, &ARDOUR_UI::engine_running));
 	engine->Halted.connect (mem_fun(*this, &ARDOUR_UI::engine_halted));
 	engine->SampleRateChanged.connect (mem_fun(*this, &ARDOUR_UI::update_sample_rate));
 
+	post_engine ();
+}
+
+void
+ARDOUR_UI::post_engine ()
+{
+	/* Things to be done once we create the AudioEngine
+	 */
+
+	check_memory_locking();
+
        	ActionManager::init ();
-	new_session_dialog = new NewSessionDialog();
-
 	_tooltips.enable();
 
-	keyboard = new Keyboard;
-
 	if (setup_windows ()) {
 		throw failed_constructor ();
 	}
 
-	if (GTK_ARDOUR::show_key_actions) {
+	/* this is the first point at which all the keybindings are available */
+
+	if (ARDOUR_COMMAND_LINE::show_key_actions) {
 		vector<string> names;
 		vector<string> paths;
 		vector<string> keys;
@@ -251,9 +314,6 @@ ARDOUR_UI::set_engine (AudioEngine& e)
 		exit (0);
 	}
 
-	/* start with timecode, metering enabled
-	*/
-	
 	blink_timeout_tag = -1;
 
 	/* the global configuration object is now valid */
@@ -272,15 +332,25 @@ ARDOUR_UI::set_engine (AudioEngine& e)
 
 	/* start the time-of-day-clock */
 	
+#ifndef GTKOSX
 	update_wall_clock ();
 	Glib::signal_timeout().connect (mem_fun(*this, &ARDOUR_UI::update_wall_clock), 60000);
+#endif
 
 	update_disk_space ();
 	update_cpu_load ();
 	update_sample_rate (engine->frame_rate());
 
-	starting.connect (mem_fun(*this, &ARDOUR_UI::startup));
-	stopping.connect (mem_fun(*this, &ARDOUR_UI::shutdown));
+	/* now start and maybe save state */
+
+	if (do_engine_start () == 0) {
+		if (session && _session_is_new) {
+			/* we need to retain initial visual 
+			   settings for a new session 
+			*/
+			session->save_state ("");
+		}
+	}
 }
 
 ARDOUR_UI::~ARDOUR_UI ()
@@ -302,6 +372,10 @@ ARDOUR_UI::~ARDOUR_UI ()
 	if (add_route_dialog) {
 		delete add_route_dialog;
 	}
+
+	if (new_session_dialog) {
+		delete new_session_dialog;
+	}
 }
 
 gint
@@ -339,7 +413,7 @@ ARDOUR_UI::configure_handler (GdkEventConfigure* conf)
 		Glib::signal_timeout().connect (mem_fun(*this, &ARDOUR_UI::configure_timeout), 100);
 		have_configure_timeout = true;
 	}
-		
+	
 	return FALSE;
 }
 
@@ -414,6 +488,11 @@ ARDOUR_UI::save_ardour_state ()
 	XMLNode* node = new XMLNode (keyboard->get_state());
 	Config->add_extra_xml (*node);
 	Config->add_extra_xml (get_transport_controllable_state());
+	if (new_session_dialog) {
+		if (new_session_dialog->engine_control.was_used()) {
+			Config->add_extra_xml (new_session_dialog->engine_control.get_state());
+		}
+	}
 	Config->save_state();
 
 	XMLNode enode(static_cast<Stateful*>(editor)->get_state());
@@ -463,10 +542,123 @@ ARDOUR_UI::update_autosave ()
         }
 }
 
+void
+ARDOUR_UI::backend_audio_error ()
+{
+	MessageDialog win (_("Ardour could not connect to JACK."),
+		     false,
+		     Gtk::MESSAGE_INFO,
+		     (Gtk::ButtonsType)(Gtk::BUTTONS_NONE));
+win.set_secondary_text(_("There are several possible reasons:\n\
+\n\
+1) JACK is not running.\n\
+2) JACK is running as another user, perhaps root.\n\
+3) There is already another client called \"ardour\".\n\
+\n\
+Please consider the possibilities, and perhaps (re)start JACK."));
+
+	win.add_button (Stock::QUIT, RESPONSE_CLOSE);
+	win.set_default_response (RESPONSE_CLOSE);
+	
+	win.show_all ();
+	win.set_position (Gtk::WIN_POS_CENTER);
+
+	if (!ARDOUR_COMMAND_LINE::no_splash) {
+		hide_splash ();
+	}
+
+	/* we just don't care about the result, but we want to block */
+
+	win.run ();
+}
+
 void
 ARDOUR_UI::startup ()
 {
-	check_memory_locking();
+	string name, path;
+	bool isnew;
+
+	new_session_dialog = new NewSessionDialog();
+	
+	/* If no session name is given: we're not loading a session yet, nor creating a new one */
+	
+	if (ARDOUR_COMMAND_LINE::session_name.length()) {
+	
+		/* Load session or start the new session dialog */
+		
+		if (Session::find_session (ARDOUR_COMMAND_LINE::session_name, path, name, isnew)) {
+			error << string_compose(_("could not load command line session \"%1\""), 
+						ARDOUR_COMMAND_LINE::session_name) << endmsg;
+			return;
+		}
+
+		if (!ARDOUR_COMMAND_LINE::new_session) {
+			
+			/* Supposed to be loading an existing session, but the session doesn't exist */
+			
+			if (isnew) {
+				error << string_compose (_("\n\nNo session named \"%1\" exists.\n"
+							   "To create it from the command line, start ardour as \"ardour --new %1"), path) 
+				      << endmsg;
+				return;
+			}
+		}
+		
+		new_session_dialog->set_session_name (name);
+		new_session_dialog->set_session_folder (Glib::path_get_basename (path));
+		_session_is_new = isnew;
+	}
+
+	hide_splash ();
+
+	bool have_backend = EngineControl::engine_running();
+	bool need_nsd;
+	bool load_needed = false;
+
+	if (have_backend) {
+
+		/* backend audio is working */
+
+		if (ARDOUR_COMMAND_LINE::session_name.empty() || ARDOUR_COMMAND_LINE::new_session) {
+			/* need NSD to get session name and other info */
+			need_nsd = true;
+		} else {
+			need_nsd = false;
+		}
+		
+	} else {
+
+		XMLNode* audio_setup = Config->extra_xml ("AudioSetup");
+		
+		if (audio_setup) {
+			new_session_dialog->engine_control.set_state (*audio_setup);
+		}
+
+		/* no backend audio, must bring up NSD to check configuration */
+		
+		need_nsd = true;
+	}
+
+	if (need_nsd) {
+
+		if (!get_session_parameters (ARDOUR_COMMAND_LINE::session_name, have_backend, ARDOUR_COMMAND_LINE::new_session)) {
+			return;
+		}
+
+	} else {
+
+		load_needed = true;
+	}
+	
+	create_engine ();
+
+	if (load_needed) {
+		if (load_session (ARDOUR_COMMAND_LINE::session_name, name)) {
+			return;
+		}
+	}
+
+	show ();
 }
 
 void
@@ -523,7 +715,6 @@ ARDOUR_UI::check_memory_locking ()
 				vbox->pack_start (hbox);
 				hbox.show_all ();
 				
-				editor->ensure_float (msg);
 				msg.run ();
 			}
 		}
@@ -562,8 +753,7 @@ If you still wish to quit, please use the\n\n\
 		session->set_deletion_in_progress ();
 	}
 	engine->stop (true);
-	Config->save_state();
-	ARDOUR_UI::config()->save_state();
+	save_ardour_state ();
 	quit ();
 }
 
@@ -638,7 +828,7 @@ ARDOUR_UI::ask_about_saving_session (const string & what)
 	return -1;
 }
 	
-gint
+int
 ARDOUR_UI::every_second ()
 {
 	update_cpu_load ();
@@ -658,6 +848,8 @@ ARDOUR_UI::every_point_one_seconds ()
 gint
 ARDOUR_UI::every_point_zero_one_seconds ()
 {
+	// august 2007: actual update frequency: 40Hz, not 100Hz
+
 	SuperRapidScreenUpdate(); /* EMIT_SIGNAL */
 	return TRUE;
 }
@@ -971,7 +1163,6 @@ ARDOUR_UI::open_session ()
 {
 	if (!check_audioengine()) {
 		return;
-		
 	}
 
 	/* popup selector window */
@@ -983,6 +1174,7 @@ ARDOUR_UI::open_session ()
 		open_session_selector = new Gtk::FileChooserDialog (_("open session"), FILE_CHOOSER_ACTION_OPEN);
 		open_session_selector->add_button (Gtk::Stock::CANCEL, Gtk::RESPONSE_CANCEL);
 		open_session_selector->add_button (Gtk::Stock::OPEN, Gtk::RESPONSE_ACCEPT);
+		open_session_selector->set_default_response(Gtk::RESPONSE_ACCEPT);
 
 		FileFilter session_filter;
 		session_filter.add_pattern ("*.ardour");
@@ -1472,21 +1664,6 @@ ARDOUR_UI::setup_theme ()
 	theme_manager->setup_theme();
 }
 
-gint
-ARDOUR_UI::start_engine ()
-{
-	if (do_engine_start () == 0) {
-		if (session && _session_is_new) {
-			/* we need to retain initial visual 
-			   settings for a new session 
-			*/
-			session->save_state ("");
-		}
-	}
-
-	return FALSE;
-}
-
 void
 ARDOUR_UI::update_clocks ()
 {
@@ -1756,6 +1933,10 @@ ARDOUR_UI::save_template ()
 	ArdourPrompter prompter (true);
 	string name;
 
+	if (!check_audioengine()) {
+		return;
+	}
+
 	prompter.set_name (X_("Prompter"));
 	prompter.set_prompt (_("Name for mix template:"));
 	prompter.set_initial_text(session->name() + _("-template"));
@@ -1776,31 +1957,36 @@ ARDOUR_UI::save_template ()
 }
 
 bool
-ARDOUR_UI::new_session (std::string predetermined_path)
+ARDOUR_UI::get_session_parameters (Glib::ustring predetermined_path, bool have_engine, bool should_be_new)
 {
 	string session_name;
 	string session_path;
+	string template_name;
 
-	if (!check_audioengine()) {
-		return false;
+	if (!loading_dialog) {
+		loading_dialog = new MessageDialog (*new_session_dialog, 
+						    _("Starting audio engine"),
+						    false,
+						    Gtk::MESSAGE_INFO,
+						    Gtk::BUTTONS_NONE);
 	}
-
+		
 	int response = Gtk::RESPONSE_NONE;
 
 	new_session_dialog->set_modal(true);
 	new_session_dialog->set_name (predetermined_path);
 	new_session_dialog->reset_recent();
+	new_session_dialog->set_position (WIN_POS_CENTER);
 	new_session_dialog->show();
+	new_session_dialog->present ();
 	new_session_dialog->set_current_page (0);
 
 	do {
+
+		new_session_dialog->set_have_engine (have_engine);
+
 	        response = new_session_dialog->run ();
 
-		if (!check_audioengine()) {
-			new_session_dialog->hide ();
-			return false;
-		}
-		
 		_session_is_new = false;
 
 		if (response == Gtk::RESPONSE_CANCEL || response == Gtk::RESPONSE_DELETE_EVENT) {
@@ -1815,10 +2001,29 @@ ARDOUR_UI::new_session (std::string predetermined_path)
 
 		        /* Clear was pressed */
 		        new_session_dialog->reset();
+			continue;
+		}
 
-		} else if (response == Gtk::RESPONSE_YES) {
+		/* first things first ... we need an audio engine running */
 
-		        /* YES  == OPEN, but there's no enum for that */
+		if (!have_engine) {
+			if (start_backend_audio ()) {
+				new_session_dialog->hide ();
+				return false;
+			}
+			have_engine = true;
+		}
+
+		loading_dialog->show_all ();
+		flush_pending ();
+		
+		create_engine ();
+
+		/* now handle possible affirmative responses */
+
+		if (response == Gtk::RESPONSE_YES) {
+
+		        /* YES == OPEN from the session selector */
 
 		        session_name = new_session_dialog->session_name();
 			
@@ -1838,21 +2043,19 @@ ARDOUR_UI::new_session (std::string predetermined_path)
 			
 		} else if (response == Gtk::RESPONSE_OK) {
 
+			/* OK == OPEN button */
+
 			session_name = new_session_dialog->session_name();
-			
-		        if (new_session_dialog->get_current_page() == 1) {
-		  
-			        /* XXX this is a bit of a hack.. 
-				   i really want the new sesion dialog to return RESPONSE_YES
-				   if we're on page 1 (the load page)
-				   Unfortunately i can't see how atm.. 
-				*/
-				
-				if (session_name.empty()) {
-					response = Gtk::RESPONSE_NONE;
-					continue;
-				} 
+		
+			if (session_name.empty()) {
+				response = Gtk::RESPONSE_NONE;
+				continue;
+			} 
 				
+			switch (new_session_dialog->get_current_page()) {
+			case 1: /* recent session selector */
+			case 2: /* audio engine control */
+
 				if (session_name[0] == '/' || 
 				    (session_name.length() > 2 && session_name[0] == '.' && session_name[1] == '/') ||
 				    (session_name.length() > 3 && session_name[0] == '.' && session_name[1] == '.' && session_name[2] == '/')) {
@@ -1861,13 +2064,15 @@ ARDOUR_UI::new_session (std::string predetermined_path)
 					session_path = new_session_dialog->session_folder();
 					load_session (session_path, session_name);
 				}
-			
-			} else {
+				break;
 
-				if (session_name.empty()) {
-					response = Gtk::RESPONSE_NONE;
-					continue;
-				} 
+			case 0: /* nominally the "new" session creator, but could be in use for an old session */
+
+				if (new_session_dialog->get_current_page() == 0 && ARDOUR_COMMAND_LINE::session_name.empty()) {
+					should_be_new = true;
+				}
+
+				/* handle what appear to be paths rather than just a name */
 
 				if (session_name[0] == '/' || 
 				    (session_name.length() > 2 && session_name[0] == '.' && session_name[1] == '/') ||
@@ -1886,8 +2091,13 @@ ARDOUR_UI::new_session (std::string predetermined_path)
 				//non-existant path. hopefully this will be fixed at some point.
 				
 				session_path = Glib::build_filename (session_path, session_name);
-						
-				if (g_file_test (session_path.c_str(), GFileTest (G_FILE_TEST_EXISTS | G_FILE_TEST_IS_DIR))) {
+				
+				if (!should_be_new) {
+
+					load_session (session_path, session_name);
+					continue; /* leaves while() loop because response != NONE */
+
+				} else if (Glib::file_test (session_path, Glib::FileTest (G_FILE_TEST_EXISTS | G_FILE_TEST_IS_DIR))) {
 
 					Glib::ustring str = string_compose (_("This session\n%1\nalready exists. Do you want to open it?"), session_path);
 
@@ -1904,6 +2114,9 @@ ARDOUR_UI::new_session (std::string predetermined_path)
 					
 					switch (msg.run()) {
 					case RESPONSE_YES:
+						new_session_dialog->hide ();
+						goto_editor_window ();
+						flush_pending ();
 						load_session (session_path, session_name);
 						goto done;
 						break;
@@ -1912,54 +2125,78 @@ ARDOUR_UI::new_session (std::string predetermined_path)
 						new_session_dialog->reset ();
 						continue;
 					}
-				}
+				} 
 
 			        _session_is_new = true;
-
-				std::string template_name = new_session_dialog->session_template_name();
 						
 				if (new_session_dialog->use_session_template()) {
-							
+
+					template_name = new_session_dialog->session_template_name();
+
+					new_session_dialog->hide ();
+					goto_editor_window ();
+					flush_pending ();
+
 					load_session (session_path, session_name, &template_name);
 			  
 				} else {
-							
+
 					uint32_t cchns;
 					uint32_t mchns;
 					AutoConnectOption iconnect;
 					AutoConnectOption oconnect;
-							
-					if (new_session_dialog->create_control_bus()) {
-						cchns = (uint32_t) new_session_dialog->control_channel_count();
-					} else {
+					uint32_t nphysin;
+					uint32_t nphysout;
+					
+					if (Profile->get_sae()) {
+
 						cchns = 0;
-					}
-							
-					if (new_session_dialog->create_master_bus()) {
-						mchns = (uint32_t) new_session_dialog->master_channel_count();
-					} else {
-						mchns = 0;
-					}
-							
-					if (new_session_dialog->connect_inputs()) {
+						mchns = 2;
 						iconnect = AutoConnectPhysical;
-					} else {
-						iconnect = AutoConnectOption (0);
-					}
-							
-					/// @todo some minor tweaks.
-							
-					if (new_session_dialog->connect_outs_to_master()) {
 						oconnect = AutoConnectMaster;
-					} else if (new_session_dialog->connect_outs_to_physical()) {
-						oconnect = AutoConnectPhysical;
+						nphysin = 0; // use all available
+						nphysout = 0; // use all available
+
 					} else {
-						oconnect = AutoConnectOption (0);
-					} 
-							
-					uint32_t nphysin = (uint32_t) new_session_dialog->input_limit_count();
-					uint32_t nphysout = (uint32_t) new_session_dialog->output_limit_count();
-							
+
+						/* get settings from advanced section of NSD */
+						
+						if (new_session_dialog->create_control_bus()) {
+							cchns = (uint32_t) new_session_dialog->control_channel_count();
+						} else {
+							cchns = 0;
+						}
+						
+						if (new_session_dialog->create_master_bus()) {
+							mchns = (uint32_t) new_session_dialog->master_channel_count();
+						} else {
+							mchns = 0;
+						}
+						
+						if (new_session_dialog->connect_inputs()) {
+							iconnect = AutoConnectPhysical;
+						} else {
+							iconnect = AutoConnectOption (0);
+						}
+						
+						/// @todo some minor tweaks.
+						
+						if (new_session_dialog->connect_outs_to_master()) {
+							oconnect = AutoConnectMaster;
+						} else if (new_session_dialog->connect_outs_to_physical()) {
+							oconnect = AutoConnectPhysical;
+						} else {
+							oconnect = AutoConnectOption (0);
+						} 
+						
+						nphysin = (uint32_t) new_session_dialog->input_limit_count();
+						nphysout = (uint32_t) new_session_dialog->output_limit_count();
+					}
+
+					new_session_dialog->hide ();
+					goto_editor_window ();
+					flush_pending ();
+
 					if (build_session (session_path,
 							   session_name,
 							   cchns,
@@ -1969,12 +2206,16 @@ ARDOUR_UI::new_session (std::string predetermined_path)
 							   nphysin,
 							   nphysout, 
 							   engine->frame_rate() * 60 * 5)) {
-
+						
 						response = Gtk::RESPONSE_NONE;
 						new_session_dialog->reset ();
 						continue;
 					}
 				}
+				break;
+
+			default:
+				break;
 			}
 		}
 		
@@ -1982,39 +2223,43 @@ ARDOUR_UI::new_session (std::string predetermined_path)
 
   done:
 	show();
-	new_session_dialog->get_window()->set_cursor();
+	loading_dialog->hide ();
 	new_session_dialog->hide();
 	return true;
 }
 
 void
-ARDOUR_UI::close_session()
+ARDOUR_UI::close_session ()
 {
 	if (!check_audioengine()) {
 		return;
 	}
 
-	unload_session();
-	new_session ();
+	unload_session (true);
+	
+	get_session_parameters ("", true, false);
 }
 
 int
 ARDOUR_UI::load_session (const string & path, const string & snap_name, string* mix_template)
 {
 	Session *new_session;
-	int x;
+	int unload_status;
+	int retval = -1;
+
 	session_loaded = false;
-	
+
 	if (!check_audioengine()) {
 		return -1;
 	}
 
-	x = unload_session ();
+	unload_status = unload_session ();
 
-	if (x < 0) {
-		return -1;
-	} else if (x > 0) {
-		return 0;
+	if (unload_status < 0) {
+		goto out;
+	} else if (unload_status > 0) {
+		retval = 0;
+		goto out;
 	}
 
 	/* if it already exists, we must have write access */
@@ -2023,17 +2268,23 @@ ARDOUR_UI::load_session (const string & path, const string & snap_name, string*
 		MessageDialog msg (*editor, _("You do not have write access to this session.\n"
 					      "This prevents the session from being loaded."));
 		msg.run ();
-		return -1;
+		goto out;
 	}
 
+	if (loading_dialog) {
+		loading_dialog->set_markup (_("Please wait while Ardour loads your session"));
+		flush_pending ();
+	}
+
+	disable_screen_updates ();
+
 	try {
 		new_session = new Session (*engine, path, snap_name, mix_template);
 	}
 
 	catch (...) {
-
 		error << string_compose(_("Session \"%1 (snapshot %2)\" did not load successfully"), path, snap_name) << endmsg;
-		return -1;
+		goto out;
 	}
 
 	connect_to_session (new_session);
@@ -2049,7 +2300,12 @@ ARDOUR_UI::load_session (const string & path, const string & snap_name, string*
 	}
 
 	editor->edit_cursor_position (true);
-	return 0;
+	enable_screen_updates ();
+	flush_pending ();
+	retval = 0;
+
+  out:
+	return retval;
 }
 
 int
@@ -2551,7 +2807,7 @@ ARDOUR_UI::cmdline_new_session (string path)
 		path = str;
 	}
 
-	new_session (path);
+	get_session_parameters (path, false, true);
 
 	_will_create_new_session_automatically = false; /* done it */
 	return FALSE; /* don't call it again */
@@ -2804,13 +3060,15 @@ ARDOUR_UI::setup_profile ()
 	if (gdk_screen_width() < 1200) {
 		Profile->set_small_screen ();
 	}
+
+	if (getenv ("ARDOUR_SAE")) {
+		Profile->set_sae ();
+		Profile->set_single_package ();
+	}
 }
 
 void
 ARDOUR_UI::audioengine_setup ()
 {
-	EngineDialog ed;
-	
-	ed.show_all ();
-	ed.run ();
 }
+
