@@ -29,6 +29,7 @@
 #include <ardour/audioregion.h>
 #include <ardour/audiosource.h>
 #include <ardour/audio_diskstream.h>
+#include <ardour/profile.h>
 #include <pbd/memento_command.h>
 #include <pbd/stacktrace.h>
 
@@ -112,7 +113,7 @@ AudioRegionView::AudioRegionView (const AudioRegionView& other)
 	UINT_TO_RGBA (other.fill_color, &r, &g, &b, &a);
 	c.set_rgb_p (r/255.0, g/255.0, b/255.0);
 	
-	init (c, false);
+	init (c, true);
 }
 
 void
@@ -121,7 +122,7 @@ AudioRegionView::init (Gdk::Color& basic_color, bool wfd)
 	// FIXME: Some redundancy here with RegionView::init.  Need to figure out
 	// where order is important and where it isn't...
 	
-	RegionView::init(basic_color, false);
+	RegionView::init (basic_color, true);
 	
 	XMLNode *node;
 
@@ -179,16 +180,18 @@ AudioRegionView::init (Gdk::Color& basic_color, bool wfd)
 	foo += ':';
 	foo += "gain";
 
-	gain_line = new AudioRegionGainLine (foo, trackview.session(), *this, *group, audio_region()->envelope());
+	if (!Profile->get_sae()) {
+		gain_line = new AudioRegionGainLine (foo, trackview.session(), *this, *group, audio_region()->envelope());
 
-	if (!(_flags & EnvelopeVisible)) {
-		gain_line->hide ();
-	} else {
-		gain_line->show ();
+		if (!(_flags & EnvelopeVisible)) {
+			gain_line->hide ();
+		} else {
+			gain_line->show ();
+		}
+
+		gain_line->reset ();
 	}
 
-	gain_line->reset ();
-
 	set_height (trackview.height);
 
 	region_muted ();
@@ -780,10 +783,14 @@ AudioRegionView::create_waves ()
 		wave_caches.push_back (WaveView::create_cache ());
 
 		if (wait_for_data) {
+
 			if (audio_region()->source(n)->peaks_ready (bind (mem_fun(*this, &AudioRegionView::peaks_ready_handler), n), data_ready_connection)) {
 				create_one_wave (n, true);
 			} else {
+				// we'll get a PeaksReady signal from the source in the future
+				// and will call create_one_wave(n) then.
 			}
+
 		} else {
 			create_one_wave (n, true);
 		}
@@ -862,7 +869,6 @@ AudioRegionView::create_one_wave (uint32_t which, bool direct)
 		/* all waves created, don't hook into peaks ready anymore */
 		data_ready_connection.disconnect ();		
 
-		if(0)
 		if (!zero_line) {
 			zero_line = new ArdourCanvas::SimpleLine (*group);
 			zero_line->property_x1() = (gdouble) 1.0;
