@@ -113,7 +113,7 @@ GainMeter::GainMeter (boost::shared_ptr<IO> io, Session& s)
 	peak_display.set_label (_("-inf"));
 	peak_display.unset_flags (Gtk::CAN_FOCUS);
 
-	meter_metric_area.set_name ("MeterMetricsStrip");
+	meter_metric_area.set_name ("AudioTrackMetrics");
 	set_size_request_to_display_given_text (meter_metric_area, "-50", 0, 0);
 
 	meter_packer.set_spacing (2);
@@ -147,39 +147,41 @@ GainMeter::GainMeter (boost::shared_ptr<IO> io, Session& s)
 	if ((r = dynamic_cast<Route*> (_io.get())) != 0) {
 
 	        /* 
-		   if we have a route (ie. we're not the click), 
+		   if we have a non-hidden route (ie. we're not the click or the auditioner), 
 		   pack some route-dependent stuff.
 		*/
 
 	        gain_display_box.pack_end (peak_display, true, true);
-
 		hbox.pack_end (meter_packer, true, true);
 
-		using namespace Menu_Helpers;
-	
-		gain_astate_menu.items().push_back (MenuElem (_("Manual"), 
-						      bind (mem_fun (*_io, &IO::set_gain_automation_state), (AutoState) Off)));
-		gain_astate_menu.items().push_back (MenuElem (_("Play"),
-						      bind (mem_fun (*_io, &IO::set_gain_automation_state), (AutoState) Play)));
-		gain_astate_menu.items().push_back (MenuElem (_("Write"),
-						      bind (mem_fun (*_io, &IO::set_gain_automation_state), (AutoState) Write)));
-		gain_astate_menu.items().push_back (MenuElem (_("Touch"),
-						      bind (mem_fun (*_io, &IO::set_gain_automation_state), (AutoState) Touch)));
-	
-		gain_astyle_menu.items().push_back (MenuElem (_("Trim")));
-		gain_astyle_menu.items().push_back (MenuElem (_("Abs")));
-	
-		gain_astate_menu.set_name ("ArdourContextMenu");
-		gain_astyle_menu.set_name ("ArdourContextMenu");
+		if (!r->hidden()) {
 
-		gain_automation_style_button.signal_button_press_event().connect (mem_fun(*this, &GainMeter::gain_automation_style_button_event), false);
-		gain_automation_state_button.signal_button_press_event().connect (mem_fun(*this, &GainMeter::gain_automation_state_button_event), false);
-		
-		r->gain_automation_curve().automation_state_changed.connect (mem_fun(*this, &GainMeter::gain_automation_state_changed));
-		r->gain_automation_curve().automation_style_changed.connect (mem_fun(*this, &GainMeter::gain_automation_style_changed));
-		fader_vbox->pack_start (gain_automation_state_button, false, false, 0);
-
-		gain_automation_state_changed ();
+			using namespace Menu_Helpers;
+			
+			gain_astate_menu.items().push_back (MenuElem (_("Manual"), 
+								      bind (mem_fun (*_io, &IO::set_gain_automation_state), (AutoState) Off)));
+			gain_astate_menu.items().push_back (MenuElem (_("Play"),
+								      bind (mem_fun (*_io, &IO::set_gain_automation_state), (AutoState) Play)));
+			gain_astate_menu.items().push_back (MenuElem (_("Write"),
+								      bind (mem_fun (*_io, &IO::set_gain_automation_state), (AutoState) Write)));
+			gain_astate_menu.items().push_back (MenuElem (_("Touch"),
+								      bind (mem_fun (*_io, &IO::set_gain_automation_state), (AutoState) Touch)));
+			
+			gain_astyle_menu.items().push_back (MenuElem (_("Trim")));
+			gain_astyle_menu.items().push_back (MenuElem (_("Abs")));
+			
+			gain_astate_menu.set_name ("ArdourContextMenu");
+			gain_astyle_menu.set_name ("ArdourContextMenu");
+			
+			gain_automation_style_button.signal_button_press_event().connect (mem_fun(*this, &GainMeter::gain_automation_style_button_event), false);
+			gain_automation_state_button.signal_button_press_event().connect (mem_fun(*this, &GainMeter::gain_automation_state_button_event), false);
+			
+			r->gain_automation_curve().automation_state_changed.connect (mem_fun(*this, &GainMeter::gain_automation_state_changed));
+			r->gain_automation_curve().automation_style_changed.connect (mem_fun(*this, &GainMeter::gain_automation_style_changed));
+			
+			fader_vbox->pack_start (gain_automation_state_button, false, false, 0);
+			gain_automation_state_changed ();
+		}
 	}
 
 	set_spacing (2);
@@ -226,6 +228,9 @@ GainMeter::set_width (Width w)
 Glib::RefPtr<Gdk::Pixmap>
 GainMeter::render_metrics (Gtk::Widget& w)
 {
+	//cerr << "name: " << w.get_name();//DEBUG
+	//cerr << " bg_red = " << w.get_style()->get_bg(Gtk::STATE_NORMAL).get_red();//DEBUG
+	//cerr << " fg_red = " << w.get_style()->get_fg(Gtk::STATE_NORMAL).get_red();//DEBUG
 	Glib::RefPtr<Gdk::Window> win (w.get_window());
 	Glib::RefPtr<Gdk::GC> fg_gc (w.get_style()->get_fg_gc (Gtk::STATE_NORMAL));
 	Glib::RefPtr<Gdk::GC> bg_gc (w.get_style()->get_bg_gc (Gtk::STATE_NORMAL));
@@ -268,13 +273,15 @@ gint
 GainMeter::meter_metrics_expose (GdkEventExpose *ev)
 {
 	static Glib::RefPtr<Gtk::Style> meter_style;
-
+	//cerr << "GainMeter::meter_metrics_expose() called on " << meter_metric_area.get_name();//DEBUG
+	//cerr << " style_changed = " << style_changed;//DEBUG
 	if (style_changed) {
 		meter_style = meter_metric_area.get_style();
+	//	cerr << " get_style() called ";//DEBUG
 	}
 	Glib::RefPtr<Gdk::Window> win (meter_metric_area.get_window());
-	Glib::RefPtr<Gdk::GC> fg_gc (meter_style->get_fg_gc (Gtk::STATE_NORMAL));
-	Glib::RefPtr<Gdk::GC> bg_gc (meter_style->get_fg_gc (Gtk::STATE_NORMAL));
+	//Glib::RefPtr<Gdk::GC> fg_gc (meter_style->get_fg_gc (Gtk::STATE_NORMAL));
+	Glib::RefPtr<Gdk::GC> bg_gc (meter_style->get_bg_gc (Gtk::STATE_INSENSITIVE));
 	GdkRectangle base_rect;
 	GdkRectangle draw_rect;
 	gint width, height;
@@ -293,12 +300,13 @@ GainMeter::meter_metrics_expose (GdkEventExpose *ev)
 		pixmap = render_metrics (meter_metric_area);
 	} else {
 		pixmap = i->second;
+	//	cerr << " re-used " << i->first;//DEBUG
 	}
 
 	gdk_rectangle_intersect (&ev->area, &base_rect, &draw_rect);
-	win->draw_rectangle (bg_gc, true, draw_rect.x, draw_rect.y, draw_rect.width, draw_rect.height);
-	win->draw_drawable (fg_gc, pixmap, draw_rect.x, draw_rect.y, draw_rect.x, draw_rect.y, draw_rect.width, draw_rect.height);
-	
+	//win->draw_rectangle (bg_gc, false, draw_rect.x, draw_rect.y, draw_rect.width, draw_rect.height);
+	win->draw_drawable (bg_gc, pixmap, draw_rect.x, draw_rect.y, draw_rect.x, draw_rect.y, draw_rect.width, draw_rect.height);
+	//cerr << endl;//DEBUG
 	style_changed = false;
 	return true;
 }
@@ -333,7 +341,7 @@ GainMeter::update_meters ()
 	for (n = 0, i = meters.begin(); i != meters.end(); ++i, ++n) {
 		if ((*i).packed) {
 			peak = _io->peak_input_power (n);
-
+			
 			(*i).meter->set (log_meter (peak));
 
 			mpeak = _io->max_peak_power(n);
@@ -623,7 +631,7 @@ void
 GainMeter::effective_gain_display ()
 {
 	gfloat value = gain_to_slider_position (_io->effective_gain());
-	
+
 	if (gain_adjustment.get_value() != value) {
 		ignore_toggle = true; 
 		gain_adjustment.set_value (value);
@@ -640,7 +648,8 @@ GainMeter::gain_changed (void *src)
 void
 GainMeter::set_meter_strip_name (const char * name)
 {
-	// meter_metric_area.set_name (name);
+	meter_metric_area.set_name (name);
+	style_changed = true;
 }
 
 void
