@@ -23,6 +23,7 @@
 #include <gtkmm/accelmap.h>
 
 #include <pbd/convert.h>
+#include <pbd/stacktrace.h>
 #include <glibmm/thread.h>
 
 #include <gtkmm2ext/gtk_ui.h>
@@ -58,15 +59,21 @@ using namespace std;
 
 using PBD::atoi;
 
+
 Mixer_UI::Mixer_UI ()
 	: Window (Gtk::WINDOW_TOPLEVEL)
 {
+	session = 0;
 	_strip_width = Wide;
 	track_menu = 0;
 	mix_group_context_menu = 0;
 	no_track_list_redisplay = false;
 	in_group_row_change = false;
 	_visible = false;
+	ignore_route_reorder = false;
+	ignore_sync = false;
+
+	Route::SyncOrderKeys.connect (mem_fun (*this, &Mixer_UI::sync_order_keys));
 
  	scroller_base.add_events (Gdk::BUTTON_PRESS_MASK|Gdk::BUTTON_RELEASE_MASK);
  	scroller_base.set_name ("MixerWindow");
@@ -92,6 +99,7 @@ Mixer_UI::Mixer_UI ()
 
 	track_model->signal_row_deleted().connect (mem_fun (*this, &Mixer_UI::track_list_delete));
 	track_model->signal_row_changed().connect (mem_fun (*this, &Mixer_UI::track_list_change));
+	track_model->signal_rows_reordered().connect (mem_fun (*this, &Mixer_UI::track_list_reorder));
 
 	CellRendererToggle* track_list_visible_cell = dynamic_cast<CellRendererToggle*>(track_display.get_column_cell_renderer (1));
 	track_list_visible_cell->property_activatable() = true;
@@ -277,7 +285,7 @@ Mixer_UI::add_strip (Session::RouteList& routes)
 		
 		strip = new MixerStrip (*this, *session, route);
 		strips.push_back (strip);
-		
+
 		if (strip->width_owner() != strip) {
 			strip->set_width (_strip_width, this);
 		}
@@ -292,11 +300,14 @@ Mixer_UI::add_strip (Session::RouteList& routes)
 		row[track_columns.visible] = strip->marked_for_display();
 		row[track_columns.route] = route;
 		row[track_columns.strip] = strip;
-		
+
+		strip->set_old_order_key (track_model->children().size() - 1);
+
 		no_track_list_redisplay = false;
 		redisplay_track_list ();
 		
 		route->name_changed.connect (bind (mem_fun(*this, &Mixer_UI::strip_name_changed), strip));
+
 		strip->GoingAway.connect (bind (mem_fun(*this, &Mixer_UI::remove_strip), strip));
 		
 		strip->signal_button_release_event().connect (bind (mem_fun(*this, &Mixer_UI::strip_button_release_event), strip));
@@ -324,6 +335,44 @@ Mixer_UI::remove_strip (MixerStrip* strip)
 	}
 }
 
+const char*
+Mixer_UI::get_order_key() 
+{
+	if (Config->get_sync_all_route_ordering()) {
+		return X_("editor");
+	} else {
+		return X_("signal");
+	}
+}
+
+
+void
+Mixer_UI::sync_order_keys ()
+{
+	vector<int> neworder;
+	TreeModel::Children rows = track_model->children();
+	TreeModel::Children::iterator ri;
+
+	if (ignore_sync || !session || (session->state_of_the_state() & Session::Loading) || rows.empty()) {
+		return;
+	}
+
+	for (ri = rows.begin(); ri != rows.end(); ++ri) {
+		neworder.push_back (0);
+	}
+
+	for (ri = rows.begin(); ri != rows.end(); ++ri) {
+		boost::shared_ptr<Route> route = (*ri)[track_columns.route];
+		MixerStrip* strip = (*ri)[track_columns.strip];
+		neworder[route->order_key (get_order_key())] = strip->old_order_key ();
+	}
+
+	ignore_route_reorder = true;
+	track_model->reorder (neworder);
+	ignore_route_reorder = false;
+}
+
+
 void
 Mixer_UI::follow_strip_selection ()
 {
@@ -359,7 +408,6 @@ Mixer_UI::strip_button_release_event (GdkEventButton *ev, MixerStrip *strip)
 void
 Mixer_UI::connect_to_session (Session* sess)
 {
-
 	session = sess;
 
 	XMLNode* node = ARDOUR_UI::instance()->mixer_settings();
@@ -509,7 +557,7 @@ Mixer_UI::set_all_audio_visibility (int tracks, bool yn)
 			continue;
 		}
 
-		AudioTrack* at = strip->audio_track();
+		boost::shared_ptr<AudioTrack> at = strip->audio_track();
 
 		switch (tracks) {
 		case 0:
@@ -568,6 +616,13 @@ Mixer_UI::hide_all_audiotracks ()
 	set_all_audio_visibility (1, false);
 }
 
+void
+Mixer_UI::track_list_reorder (const TreeModel::Path& path, const TreeModel::iterator& iter, int* new_order)
+{
+	session->set_remote_control_ids();
+	redisplay_track_list ();
+}
+
 void
 Mixer_UI::track_list_change (const Gtk::TreeModel::Path& path,const Gtk::TreeModel::iterator& iter)
 {
@@ -602,10 +657,17 @@ Mixer_UI::redisplay_track_list ()
 		}
 
 		bool visible = (*i)[track_columns.visible];
+		
+		boost::shared_ptr<Route> route = (*i)[track_columns.route];
 
 		if (visible) {
 			strip->set_marked_for_display (true);
-			strip->route()->set_order_key (N_("signal"), order);
+
+			if (!ignore_route_reorder) {
+				strip->route()->set_order_key (get_order_key(), order);
+			} 
+
+			strip->set_old_order_key (order);
 
 			if (strip->packed()) {
 
@@ -638,12 +700,18 @@ Mixer_UI::redisplay_track_list ()
 			}
 		}
 	}
+	
+	if (Config->get_sync_all_route_ordering() && !ignore_route_reorder) {
+		ignore_sync = true;
+		Route::SyncOrderKeys (); // EMIT SIGNAL
+		ignore_sync = false;
+	}
 }
 
 struct SignalOrderRouteSorter {
     bool operator() (boost::shared_ptr<Route> a, boost::shared_ptr<Route> b) {
 	    /* use of ">" forces the correct sort order */
-	    return a->order_key ("signal") < b->order_key ("signal");
+	    return a->order_key (Mixer_UI::get_order_key()) < b->order_key (Mixer_UI::get_order_key());
     }
 };
 
