@@ -339,8 +339,10 @@ RedirectBox::redirect_button_release_event (GdkEventButton *ev)
 		show_redirect_menu(ev->time);
 		ret = true;
 
-	} else if (redirect && (ev->button == 2) && (ev->state == Gdk::BUTTON2_MASK)) {
+	} else if (redirect && (ev->button == 2) && (Keyboard::no_modifier_keys_pressed (ev) && ((ev->state & Gdk::BUTTON2_MASK) == Gdk::BUTTON2_MASK))) {
 		
+		/* button2-click with no modifiers */
+
 		redirect->set_active (!redirect->active(), this);
 		ret = true;
 
@@ -395,13 +397,12 @@ RedirectBox::insert_plugin_chosen (boost::shared_ptr<Plugin> plugin)
 
 		boost::shared_ptr<Redirect> redirect (new PluginInsert (_session, plugin, _placement));
 		
-		redirect->active_changed.connect (bind (mem_fun (*this, &RedirectBox::show_redirect_active_r), boost::weak_ptr<Redirect>(redirect)));
-
 		uint32_t err_streams;
 
 		if (_route->add_redirect (redirect, this, &err_streams)) {
 			weird_plugin_dialog (*plugin, err_streams, _route);
-			// XXX SHAREDPTR delete plugin here .. do we even need to care? 
+		} else {
+			redirect->active_changed.connect (bind (mem_fun (*this, &RedirectBox::show_redirect_active_r), boost::weak_ptr<Redirect>(redirect)));
 		}
 	}
 }
@@ -648,14 +649,14 @@ RedirectBox::show_redirect_active_r (Redirect* r, void *src, boost::weak_ptr<Red
 void
 RedirectBox::show_redirect_active (boost::weak_ptr<Redirect> weak_redirect)
 {
-	ENSURE_GUI_THREAD(bind (mem_fun(*this, &RedirectBox::show_redirect_active), weak_redirect));
-	
 	boost::shared_ptr<Redirect> redirect (weak_redirect.lock());
 	
 	if (!redirect) {
 		return;
 	}
 
+	ENSURE_GUI_THREAD(bind (mem_fun(*this, &RedirectBox::show_redirect_active), weak_redirect));
+	
 	Gtk::TreeModel::Children children = model->children();
 	Gtk::TreeModel::Children::iterator iter = children.begin();
 
