@@ -65,17 +65,30 @@ class LadspaPlugin : public ARDOUR::Plugin
 	std::set<uint32_t> automatable() const;
 	uint32_t nth_parameter (uint32_t port, bool& ok) const;
 	void activate () { 
+		if (was_activated)
+			return;
+
 		if (descriptor->activate) {
 			descriptor->activate (handle);
 		}
+
 		was_activated = true;
 	}
 	void deactivate () {
-		if (descriptor->deactivate) 
+		if (!was_activated)
+			return;
+
+		if (descriptor->deactivate) {
 			descriptor->deactivate (handle);
+		}
+	
+		was_activated = false;
 	}
 	void cleanup () {
-		if (was_activated && descriptor->cleanup) {
+		activate();
+		deactivate();
+
+		if (descriptor->cleanup) {
 			descriptor->cleanup (handle);
 		}
 	}
