@@ -15,79 +15,103 @@
     along with this program; if not, write to the Free Software
     Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
 
-    $Id: audio_track.cc,v 1.75 2005/12/28 21:38:52 essej Exp $
 */
 
-#include <sigc++/rettype.h>
+#include <sigc++/retype.h>
+#include <sigc++/retype_return.h>
 #include <sigc++/bind.h>
 
+#include <pbd/error.h>
+#include <pbd/enumwriter.h>
+
 #include <ardour/audio_track.h>
-#include <ardour/diskstream.h>
+#include <ardour/audio_diskstream.h>
 #include <ardour/session.h>
 #include <ardour/redirect.h>
 #include <ardour/audioregion.h>
+#include <ardour/audiosource.h>
+#include <ardour/region_factory.h>
 #include <ardour/route_group_specialized.h>
 #include <ardour/insert.h>
 #include <ardour/audioplaylist.h>
+#include <ardour/playlist_factory.h>
 #include <ardour/panner.h>
 #include <ardour/utils.h>
 
 #include "i18n.h"
 
 using namespace std;
-using namespace SigC;
 using namespace ARDOUR;
+using namespace PBD;
 
-AudioTrack::AudioTrack (Session& sess, string name, Route::Flag flag)
-	: Route (sess, name, 1, -1, -1, -1, flag),
-	  diskstream (0),
-	  _midi_rec_enable_control (*this, _session.midi_port())
+AudioTrack::AudioTrack (Session& sess, string name, Route::Flag flag, TrackMode mode)
+	: Track (sess, name, flag, mode)
 {
-	DiskStream::Flag dflags = DiskStream::Flag (0);
+	AudioDiskstream::Flag dflags = AudioDiskstream::Flag (0);
 
 	if (_flags & Hidden) {
-		dflags = DiskStream::Flag (dflags | DiskStream::Hidden);
+		dflags = AudioDiskstream::Flag (dflags | AudioDiskstream::Hidden);
 	} else {
-		dflags = DiskStream::Flag (dflags | DiskStream::Recordable);
+		dflags = AudioDiskstream::Flag (dflags | AudioDiskstream::Recordable);
 	}
 
-	DiskStream* ds = new DiskStream (_session, name, dflags);
-	
-	set_diskstream (*ds, this);
-	
-	_declickable = true;
-	_freeze_record.state = NoFreeze;
-	_saved_meter_point = _meter_point;
+	if (mode == Destructive) {
+		dflags = AudioDiskstream::Flag (dflags | AudioDiskstream::Destructive);
+	}
 
-	// we do this even though Route already did it in it's init
-	reset_midi_control (_session.midi_port(), _session.get_midi_control());
+	boost::shared_ptr<AudioDiskstream> ds (new AudioDiskstream (_session, name, dflags));
 	
+	_session.add_diskstream (ds);
+
+	set_diskstream (boost::dynamic_pointer_cast<AudioDiskstream> (ds), this);
 }
 
 AudioTrack::AudioTrack (Session& sess, const XMLNode& node)
-	: Route (sess, "to be renamed", 0, 0, -1, -1),
-	  diskstream (0),
-	  _midi_rec_enable_control (*this, _session.midi_port())
+	: Track (sess, node)
 {
-	_freeze_record.state = NoFreeze;
-	set_state (node);
-	_declickable = true;
-	_saved_meter_point = _meter_point;
-
-	// we do this even though Route already did it in it's init
-	reset_midi_control (_session.midi_port(), _session.get_midi_control());
+	_set_state (node, false);
 }
 
 AudioTrack::~AudioTrack ()
 {
-	if (diskstream) {
-		diskstream->unref();
+}
+
+int
+AudioTrack::set_mode (TrackMode m)
+{
+	if (m != _mode) {
+
+		if (_diskstream->set_destructive (m == Destructive)) {
+			return -1;
+		}
+
+		_mode = m;
+		
+		TrackModeChanged (); /* EMIT SIGNAL */
+	}
+
+	return 0;
+}
+
+bool
+AudioTrack::can_use_mode (TrackMode m, bool& bounce_required)
+{
+	switch (m) {
+	case Normal:
+		bounce_required = false;
+		return true;
+		
+	case Destructive:
+	default:
+		return _diskstream->can_become_destructive (bounce_required);
 	}
 }
 
 int
 AudioTrack::deprecated_use_diskstream_connections ()
 {
+	boost::shared_ptr<AudioDiskstream> diskstream = audio_diskstream();
+
 	if (diskstream->deprecated_io_node == 0) {
 		return 0;
 	}
@@ -113,14 +137,22 @@ AudioTrack::deprecated_use_diskstream_connections ()
 		Connection* c = _session.connection_by_name (prop->value());
 		
 		if (c == 0) {
-			error << compose(_("Unknown connection \"%1\" listed for input of %2"), prop->value(), _name) << endmsg;
+		  	error << string_compose(_("Unknown connection \"%1\" listed for input of %2"), prop->value(), _name) << endmsg;
+
+			string replacement_connection;
+
+			if (prop->value().find ('+') != string::npos) {
+				replacement_connection = _("in 1+2");
+			} else {
+				replacement_connection = _("in 1");
+			}
 			
-			if ((c = _session.connection_by_name (_("in 1"))) == 0) {
-				error << _("No input connections available as a replacement")
-				      << endmsg;
+			if ((c = _session.connection_by_name (replacement_connection)) == 0) {
+			  	error << _("No input connections available as a replacement")
+			        << endmsg;
 				return -1;
 			} else {
-				info << compose (_("Connection %1 was not available - \"in 1\" used instead"), prop->value())
+			  	info << string_compose (_("Connection %1 was not available - \"%2\" used instead"), prop->value(), replacement_connection)
 				     << endmsg;
 			}
 		}
@@ -129,7 +161,7 @@ AudioTrack::deprecated_use_diskstream_connections ()
 
 	} else if ((prop = node.property ("inputs")) != 0) {
 		if (set_inputs (prop->value())) {
-			error << compose(_("improper input channel list in XML node (%1)"), prop->value()) << endmsg;
+		  	error << string_compose(_("improper input channel list in XML node (%1)"), prop->value()) << endmsg;
 			return -1;
 		}
 	}
@@ -138,31 +170,28 @@ AudioTrack::deprecated_use_diskstream_connections ()
 }
 
 int
-AudioTrack::set_diskstream (DiskStream& ds, void *src)
+AudioTrack::set_diskstream (boost::shared_ptr<AudioDiskstream> ds, void *src)
 {
-	if (diskstream) {
-		diskstream->unref();
-	}
+	_diskstream = ds;
+	_diskstream->set_io (*this);
+	_diskstream->set_destructive (_mode == Destructive);
 
-	diskstream = &ds.ref();
-	diskstream->set_io (*this);
-
-	if (diskstream->deprecated_io_node) {
+	if (audio_diskstream()->deprecated_io_node) {
 
 		if (!connecting_legal) {
-			ConnectingLegal.connect (slot (*this, &AudioTrack::deprecated_use_diskstream_connections));
+			ConnectingLegal.connect (mem_fun (*this, &AudioTrack::deprecated_use_diskstream_connections));
 		} else {
 			deprecated_use_diskstream_connections ();
 		}
 	}
 
-	diskstream->set_record_enabled (false, this);
-	diskstream->monitor_input (false);
+	_diskstream->set_record_enabled (false);
+	_diskstream->monitor_input (false);
 
 	ic_connection.disconnect();
-	ic_connection = input_changed.connect (slot (*diskstream, &DiskStream::handle_input_change));
+	ic_connection = input_changed.connect (mem_fun (*_diskstream, &Diskstream::handle_input_change));
 
-	diskstream_changed (src); /* EMIT SIGNAL */
+	DiskstreamChanged (); /* EMIT SIGNAL */
 
 	return 0;
 }	
@@ -170,132 +199,59 @@ AudioTrack::set_diskstream (DiskStream& ds, void *src)
 int 
 AudioTrack::use_diskstream (string name)
 {
-	DiskStream *dstream;
+	boost::shared_ptr<AudioDiskstream> dstream;
 
-	if ((dstream = _session.diskstream_by_name (name)) == 0) {
-		error << compose(_("AudioTrack: diskstream \"%1\" not known by session"), name) << endmsg;
+	if ((dstream = boost::dynamic_pointer_cast<AudioDiskstream>(_session.diskstream_by_name (name))) == 0) {
+		error << string_compose(_("AudioTrack: audio diskstream \"%1\" not known by session"), name) << endmsg;
 		return -1;
 	}
 	
-	return set_diskstream (*dstream, this);
+	return set_diskstream (dstream, this);
 }
 
 int 
-AudioTrack::use_diskstream (id_t id)
+AudioTrack::use_diskstream (const PBD::ID& id)
 {
-	DiskStream *dstream;
+	boost::shared_ptr<AudioDiskstream> dstream;
 
-	if ((dstream = _session.diskstream_by_id (id)) == 0) {
-		error << compose(_("AudioTrack: diskstream \"%1\" not known by session"), id) << endmsg;
+	if ((dstream = boost::dynamic_pointer_cast<AudioDiskstream> (_session.diskstream_by_id (id))) == 0) {
+	  	error << string_compose(_("AudioTrack: audio diskstream \"%1\" not known by session"), id) << endmsg;
 		return -1;
 	}
 	
-	return set_diskstream (*dstream, this);
+	return set_diskstream (dstream, this);
 }
 
-bool
-AudioTrack::record_enabled () const
+boost::shared_ptr<AudioDiskstream>
+AudioTrack::audio_diskstream() const
 {
-	return diskstream->record_enabled ();
-}
-
-void
-AudioTrack::set_record_enable (bool yn, void *src)
-{
-	if (_freeze_record.state == Frozen) {
-		return;
-	}
-
-	if (_mix_group && src != _mix_group && _mix_group->is_active()) {
-		_mix_group->apply (&AudioTrack::set_record_enable, yn, _mix_group);
-		return;
-	}
-
-	/* keep track of the meter point as it was before we rec-enabled */
-
-	if (!diskstream->record_enabled()) {
-		_saved_meter_point = _meter_point;
-	}
-	
-	diskstream->set_record_enabled (yn, src);
-
-	if (diskstream->record_enabled()) {
-		set_meter_point (MeterInput, this);
-	} else {
-		set_meter_point (_saved_meter_point, this);
-	}
-
-	if (_session.get_midi_feedback()) {
-		_midi_rec_enable_control.send_feedback (record_enabled());
-	}
-
-}
-
-void
-AudioTrack::set_meter_point (MeterPoint p, void *src)
-{
-	Route::set_meter_point (p, src);
-}
-
-XMLNode&
-AudioTrack::state(bool full_state)
-{
-	XMLNode& track_state (Route::state (full_state));
-	char buf[64];
-
-	/* we don't return diskstream state because we don't
-	   own the diskstream exclusively. control of the diskstream
-	   state is ceded to the Session, even if we create the
-	   diskstream.
-	*/
-
-	snprintf (buf, sizeof (buf), "%" PRIu64, diskstream->id());
-	track_state.add_property ("diskstream-id", buf);
-
-	return track_state;
+	return boost::dynamic_pointer_cast<AudioDiskstream>(_diskstream);
 }
 
 int
 AudioTrack::set_state (const XMLNode& node)
+{
+	return _set_state (node, true);
+}
+
+int
+AudioTrack::_set_state (const XMLNode& node, bool call_base)
 {
 	const XMLProperty *prop;
 	XMLNodeConstIterator iter;
-	XMLNodeList midi_kids;
 
-	if (Route::set_state (node)) {
-		return -1;
-	}
-
-	midi_kids = node.children ("MIDI");
-	
-	for (iter = midi_kids.begin(); iter != midi_kids.end(); ++iter) {
-	
-		XMLNodeList kids;
-		XMLNodeConstIterator miter;
-		XMLNode*    child;
-
-		kids = (*iter)->children ();
-
-		for (miter = kids.begin(); miter != kids.end(); ++miter) {
-
-			child =* miter;
-
-			if (child->name() == "rec_enable") {
-			
-				MIDI::eventType ev = MIDI::on; /* initialize to keep gcc happy */
-				MIDI::byte additional = 0;  /* ditto */
-				MIDI::channel_t chn = 0;    /* ditto */
-
-				if (get_midi_node_info (child, ev, chn, additional)) {
-					_midi_rec_enable_control.set_control_type (chn, ev, additional);
-				} else {
-					error << compose(_("MIDI rec_enable control specification for %1 is incomplete, so it has been ignored"), _name) << endmsg;
-				}
-			}
+	if (call_base) {
+		if (Route::_set_state (node, call_base)) {
+			return -1;
 		}
 	}
 
-	
+	if ((prop = node.property (X_("mode"))) != 0) {
+		_mode = TrackMode (string_2_enum (prop->value(), _mode));
+	} else {
+		_mode = Normal;
+	}
+
 	if ((prop = node.property ("diskstream-id")) == 0) {
 		
 		/* some old sessions use the diskstream name rather than the ID */
@@ -312,38 +268,52 @@ AudioTrack::set_state (const XMLNode& node)
 
 	} else {
 		
-		id_t id = strtoull (prop->value().c_str(), 0, 10);
+		PBD::ID id (prop->value());
 		
 		if (use_diskstream (id)) {
 			return -1;
 		}
 	}
 
+
+	XMLNodeList nlist;
+	XMLNodeConstIterator niter;
+	XMLNode *child;
+
+	nlist = node.children();
+	for (niter = nlist.begin(); niter != nlist.end(); ++niter){
+		child = *niter;
+
+		if (child->name() == X_("recenable")) {
+			_rec_enable_control.set_state (*child);
+			_session.add_controllable (&_rec_enable_control);
+		}
+	}
+
 	pending_state = const_cast<XMLNode*> (&node);
 
-	_session.StateReady.connect (slot (*this, &AudioTrack::set_state_part_two));
+	_session.StateReady.connect (mem_fun (*this, &AudioTrack::set_state_part_two));
 
 	return 0;
 }
 
 XMLNode& 
-AudioTrack::get_state()
+AudioTrack::state(bool full_state)
 {
-	XMLNode& root (Route::get_state());
+	XMLNode& root (Route::state(full_state));
 	XMLNode* freeze_node;
-	char buf[32];
+	char buf[64];
 
 	if (_freeze_record.playlist) {
 		XMLNode* inode;
 
 		freeze_node = new XMLNode (X_("freeze-info"));
 		freeze_node->add_property ("playlist", _freeze_record.playlist->name());
-		snprintf (buf, sizeof (buf), "%d", (int) _freeze_record.state);
-		freeze_node->add_property ("state", buf);
+		freeze_node->add_property ("state", enum_2_string (_freeze_record.state));
 
 		for (vector<FreezeRecordInsertInfo*>::iterator i = _freeze_record.insert_info.begin(); i != _freeze_record.insert_info.end(); ++i) {
 			inode = new XMLNode (X_("insert"));
-			snprintf (buf, sizeof (buf), "%" PRIu64, (*i)->id);
+			(*i)->id.print (buf, sizeof (buf));
 			inode->add_property (X_("id"), buf);
 			inode->add_child_copy ((*i)->state);
 		
@@ -356,41 +326,23 @@ AudioTrack::get_state()
 	/* Alignment: act as a proxy for the diskstream */
 	
 	XMLNode* align_node = new XMLNode (X_("alignment"));
-	switch (diskstream->alignment_style()) {
-	case ExistingMaterial:
-		snprintf (buf, sizeof (buf), X_("existing"));
-		break;
-	case CaptureTime:
-		snprintf (buf, sizeof (buf), X_("capture"));
-		break;
-	}
-	align_node->add_property (X_("style"), buf);
+	AlignStyle as = _diskstream->alignment_style ();
+	align_node->add_property (X_("style"), enum_2_string (as));
 	root.add_child_nocopy (*align_node);
 
-	/* MIDI control */
+	root.add_property (X_("mode"), enum_2_string (_mode));
 
-	MIDI::channel_t chn;
-	MIDI::eventType ev;
-	MIDI::byte      additional;
-	XMLNode*        midi_node = 0;
-	XMLNode*        child;
-	XMLNodeList     midikids;
+	/* we don't return diskstream state because we don't
+	   own the diskstream exclusively. control of the diskstream
+	   state is ceded to the Session, even if we create the
+	   diskstream.
+	*/
 
-	midikids = root.children ("MIDI");
-	if (!midikids.empty()) {
-		midi_node = midikids.front();
-	}
-	else {
-		midi_node = root.add_child ("MIDI");
-	}
-		
-	if (_midi_rec_enable_control.get_control_info (chn, ev, additional) && midi_node) {
+	_diskstream->id().print (buf, sizeof (buf));
+	root.add_property ("diskstream-id", buf);
 
-		child = midi_node->add_child ("rec_enable");
-		set_midi_node_info (child, ev, chn, additional);
-	}
+	root.add_child_nocopy (_rec_enable_control.get_state());
 
-	
 	return root;
 }
 
@@ -421,18 +373,18 @@ AudioTrack::set_state_part_two ()
 		_freeze_record.insert_info.clear ();
 		
 		if ((prop = fnode->property (X_("playlist"))) != 0) {
-			Playlist* pl = _session.playlist_by_name (prop->value());
+			boost::shared_ptr<Playlist> pl = _session.playlist_by_name (prop->value());
 			if (pl) {
-				_freeze_record.playlist = dynamic_cast<AudioPlaylist*> (pl);
+				_freeze_record.playlist = boost::dynamic_pointer_cast<AudioPlaylist> (pl);
 			} else {
-				_freeze_record.playlist = 0;
+				_freeze_record.playlist.reset ();
 				_freeze_record.state = NoFreeze;
 			return;
 			}
 		}
 		
 		if ((prop = fnode->property (X_("state"))) != 0) {
-			_freeze_record.state = (FreezeState) atoi (prop->value().c_str());
+			_freeze_record.state = FreezeState (string_2_enum (prop->value(), _freeze_record.state));
 		}
 		
 		XMLNodeConstIterator citer;
@@ -447,9 +399,9 @@ AudioTrack::set_state_part_two ()
 				continue;
 			}
 			
-			FreezeRecordInsertInfo* frii = new FreezeRecordInsertInfo (*((*citer)->children().front()));
-			frii->insert = 0;
-			sscanf (prop->value().c_str(), "%llu", &frii->id);
+			FreezeRecordInsertInfo* frii = new FreezeRecordInsertInfo (*((*citer)->children().front()),
+										   boost::shared_ptr<Insert>());
+			frii->id = prop->value ();
 			_freeze_record.insert_info.push_back (frii);
 		}
 	}
@@ -459,11 +411,21 @@ AudioTrack::set_state_part_two ()
 	if ((fnode = find_named_node (*pending_state, X_("alignment"))) != 0) {
 
 		if ((prop = fnode->property (X_("style"))) != 0) {
-			if (prop->value() == "existing") {
-				diskstream->set_persistent_align_style (ExistingMaterial);
-			} else if (prop->value() == "capture") {
-				diskstream->set_persistent_align_style (CaptureTime);
+
+			/* fix for older sessions from before EnumWriter */
+
+			string pstr;
+
+			if (prop->value() == "capture") {
+				pstr = "CaptureTime";
+			} else if (prop->value() == "existing") {
+				pstr = "ExistingMaterial";
+			} else {
+				pstr = prop->value();
 			}
+
+			AlignStyle as = AlignStyle (string_2_enum (pstr, as));
+			_diskstream->set_persistent_align_style (as);
 		}
 	}
 	return;
@@ -472,18 +434,18 @@ AudioTrack::set_state_part_two ()
 uint32_t
 AudioTrack::n_process_buffers ()
 {
-	return max ((uint32_t) diskstream->n_channels(), redirect_max_outs);
+	return max ((uint32_t) _diskstream->n_channels(), redirect_max_outs);
 }
 
 void
-AudioTrack::passthru_silence (jack_nframes_t start_frame, jack_nframes_t end_frame, jack_nframes_t nframes, jack_nframes_t offset, int declick, bool meter)
+AudioTrack::passthru_silence (nframes_t start_frame, nframes_t end_frame, nframes_t nframes, nframes_t offset, int declick, bool meter)
 {
 	uint32_t nbufs = n_process_buffers ();
 	process_output_buffers (_session.get_silent_buffers (nbufs), nbufs, start_frame, end_frame, nframes, offset, true, declick, meter);
 }
 
 int 
-AudioTrack::no_roll (jack_nframes_t nframes, jack_nframes_t start_frame, jack_nframes_t end_frame, jack_nframes_t offset, 
+AudioTrack::no_roll (nframes_t nframes, nframes_t start_frame, nframes_t end_frame, nframes_t offset, 
 		     bool session_state_changing, bool can_record, bool rec_monitors_input)
 {
 	if (n_outputs() == 0) {
@@ -503,7 +465,7 @@ AudioTrack::no_roll (jack_nframes_t nframes, jack_nframes_t start_frame, jack_nf
 		return 0;
 	}
 
-	diskstream->check_record_status (start_frame, nframes, can_record);
+	audio_diskstream()->check_record_status (start_frame, nframes, can_record);
 
 	bool send_silence;
 	
@@ -515,18 +477,18 @@ AudioTrack::no_roll (jack_nframes_t nframes, jack_nframes_t start_frame, jack_nf
 		send_silence = true;
 	} else {
 
-		if (_session.get_auto_input()) {
-			if (Config->get_no_sw_monitoring()) {
-				send_silence = true;
-			} else {
+		if (Config->get_auto_input()) {
+			if (Config->get_monitoring_model() == SoftwareMonitoring) {
 				send_silence = false;
+			} else {
+				send_silence = true;
 			}
 		} else {
-			if (diskstream->record_enabled()) {
-				if (Config->get_no_sw_monitoring()) {
-					send_silence = true;
-				} else {
+			if (_diskstream->record_enabled()) {
+				if (Config->get_monitoring_model() == SoftwareMonitoring) {
 					send_silence = false;
+				} else {
+					send_silence = true;
 				}
 			} else {
 				send_silence = true;
@@ -563,22 +525,24 @@ AudioTrack::no_roll (jack_nframes_t nframes, jack_nframes_t start_frame, jack_nf
 }
 
 int
-AudioTrack::roll (jack_nframes_t nframes, jack_nframes_t start_frame, jack_nframes_t end_frame, jack_nframes_t offset, int declick,
+AudioTrack::roll (nframes_t nframes, nframes_t start_frame, nframes_t end_frame, nframes_t offset, int declick,
 		  bool can_record, bool rec_monitors_input)
 {
 	int dret;
 	Sample* b;
 	Sample* tmpb;
-	jack_nframes_t transport_frame;
-
+	nframes_t transport_frame;
+	boost::shared_ptr<AudioDiskstream> diskstream = audio_diskstream();
+	
 	{
-		TentativeRWLockMonitor lm (redirect_lock, false, __LINE__, __FILE__);
+		Glib::RWLock::ReaderLock lm (redirect_lock, Glib::TRY_LOCK);
 		if (lm.locked()) {
 			// automation snapshot can also be called from the non-rt context
 			// and it uses the redirect list, so we take the lock out here
 			automation_snapshot (start_frame);
 		}
 	}
+
 	
 	if (n_outputs() == 0 && _redirects.empty()) {
 		return 0;
@@ -615,7 +579,7 @@ AudioTrack::roll (jack_nframes_t nframes, jack_nframes_t start_frame, jack_nfram
 		just_meter_input (start_frame, end_frame, nframes, offset);
 	}
 
-	if (diskstream->record_enabled() && !can_record && !_session.get_auto_input()) {
+	if (diskstream->record_enabled() && !can_record && !Config->get_auto_input()) {
 
 		/* not actually recording, but we want to hear the input material anyway,
 		   at least potentially (depending on monitoring options)
@@ -656,14 +620,14 @@ AudioTrack::roll (jack_nframes_t nframes, jack_nframes_t start_frame, jack_nfram
 		/* don't waste time with automation if we're recording or we've just stopped (yes it can happen) */
 
 		if (!diskstream->record_enabled() && _session.transport_rolling()) {
-			TentativeLockMonitor am (automation_lock, __LINE__, __FILE__);
+			Glib::Mutex::Lock am (automation_lock, Glib::TRY_LOCK);
 			
 			if (am.locked() && gain_automation_playback()) {
 				apply_gain_automation = _gain_automation_curve.rt_safe_get_vector (start_frame, end_frame, _session.gain_automation_buffer(), nframes);
 			}
 		}
 
-		process_output_buffers (bufs, limit, start_frame, end_frame, nframes, offset, (!_session.get_record_enabled() || _session.get_recording_plugins()), declick, (_meter_point != MeterInput));
+		process_output_buffers (bufs, limit, start_frame, end_frame, nframes, offset, (!_session.get_record_enabled() || !Config->get_do_not_record_plugins()), declick, (_meter_point != MeterInput));
 		
 	} else {
 		/* problem with the diskstream; just be quiet for a bit */
@@ -674,7 +638,7 @@ AudioTrack::roll (jack_nframes_t nframes, jack_nframes_t start_frame, jack_nfram
 }
 
 int
-AudioTrack::silent_roll (jack_nframes_t nframes, jack_nframes_t start_frame, jack_nframes_t end_frame, jack_nframes_t offset, 
+AudioTrack::silent_roll (nframes_t nframes, nframes_t start_frame, nframes_t end_frame, nframes_t offset, 
 			 bool can_record, bool rec_monitors_input)
 {
 	if (n_outputs() == 0 && _redirects.empty()) {
@@ -691,31 +655,11 @@ AudioTrack::silent_roll (jack_nframes_t nframes, jack_nframes_t start_frame, jac
 
 	silence (nframes, offset);
 
-	return diskstream->process (_session.transport_frame() + offset, nframes, offset, can_record, rec_monitors_input);
-}
-
-void
-AudioTrack::toggle_monitor_input ()
-{
-	for (vector<Port*>::iterator i = _inputs.begin(); i != _inputs.end(); ++i) {
-		(*i)->request_monitor_input(!(*i)->monitoring_input());
-	}
+	return audio_diskstream()->process (_session.transport_frame() + offset, nframes, offset, can_record, rec_monitors_input);
 }
 
 int
-AudioTrack::set_name (string str, void *src)
-{
-	if (record_enabled() && _session.actively_recording()) {
-		/* this messes things up if done while recording */
-		return -1;
-	}
-
-	diskstream->set_name (str, src);
-	return IO::set_name (str, src);
-}
-
-int
-AudioTrack::export_stuff (vector<Sample*>& buffers, uint32_t nbufs, jack_nframes_t start, jack_nframes_t nframes)
+AudioTrack::export_stuff (vector<Sample*>& buffers, uint32_t nbufs, nframes_t start, nframes_t nframes)
 {
 	gain_t  gain_automation[nframes];
 	gain_t  gain_buffer[nframes];
@@ -725,10 +669,14 @@ AudioTrack::export_stuff (vector<Sample*>& buffers, uint32_t nbufs, jack_nframes
 	gain_t this_gain = _gain;
 	vector<Sample*>::iterator bi;
 	Sample * b;
+	boost::shared_ptr<AudioDiskstream> diskstream = audio_diskstream();
 	
-	RWLockMonitor rlock (redirect_lock, false, __LINE__, __FILE__);
-		
-	if (diskstream->playlist()->read (buffers[0], mix_buffer, gain_buffer, start, nframes) != nframes) {
+	Glib::RWLock::ReaderLock rlock (redirect_lock);
+
+	boost::shared_ptr<AudioPlaylist> apl = boost::dynamic_pointer_cast<AudioPlaylist>(diskstream->playlist());
+	assert(apl);
+
+	if (apl->read (buffers[0], mix_buffer, gain_buffer, start, nframes) != nframes) {
 		return -1;
 	}
 
@@ -738,7 +686,7 @@ AudioTrack::export_stuff (vector<Sample*>& buffers, uint32_t nbufs, jack_nframes
 	++bi;
 	for (; bi != buffers.end(); ++bi, ++n) {
 		if (n < diskstream->n_channels()) {
-			if (diskstream->playlist()->read ((*bi), mix_buffer, gain_buffer, start, nframes, n) != nframes) {
+			if (apl->read ((*bi), mix_buffer, gain_buffer, start, nframes, n) != nframes) {
 				return -1;
 			}
 			b = (*bi);
@@ -755,9 +703,9 @@ AudioTrack::export_stuff (vector<Sample*>& buffers, uint32_t nbufs, jack_nframes
 	*/
 	
 	for (i = _redirects.begin(); i != _redirects.end(); ++i) {
-		Insert *insert;
+		boost::shared_ptr<Insert> insert;
 		
-		if ((insert = dynamic_cast<Insert*>(*i)) != 0) {
+		if ((insert = boost::dynamic_pointer_cast<Insert>(*i)) != 0) {
 			switch (insert->placement()) {
 			case PreFader:
 				insert->run (buffers, nbufs, nframes, 0);
@@ -775,7 +723,7 @@ AudioTrack::export_stuff (vector<Sample*>& buffers, uint32_t nbufs, jack_nframes
 
 		for (bi = buffers.begin(); bi != buffers.end(); ++bi) {
 			Sample *b = *bi;
-			for (jack_nframes_t n = 0; n < nframes; ++n) {
+			for (nframes_t n = 0; n < nframes; ++n) {
 				b[n] *= gain_automation[n];
 			}
 		}
@@ -784,7 +732,7 @@ AudioTrack::export_stuff (vector<Sample*>& buffers, uint32_t nbufs, jack_nframes
 
 		for (bi = buffers.begin(); bi != buffers.end(); ++bi) {
 			Sample *b = *bi;
-			for (jack_nframes_t n = 0; n < nframes; ++n) {
+			for (nframes_t n = 0; n < nframes; ++n) {
 				b[n] *= this_gain;
 			}
 		}
@@ -793,9 +741,9 @@ AudioTrack::export_stuff (vector<Sample*>& buffers, uint32_t nbufs, jack_nframes
 	if (post_fader_work) {
 
 		for (i = _redirects.begin(); i != _redirects.end(); ++i) {
-			PluginInsert *insert;
+			boost::shared_ptr<PluginInsert> insert;
 			
-			if ((insert = dynamic_cast<PluginInsert*>(*i)) != 0) {
+			if ((insert = boost::dynamic_pointer_cast<PluginInsert>(*i)) != 0) {
 				switch ((*i)->placement()) {
 				case PreFader:
 					break;
@@ -810,66 +758,42 @@ AudioTrack::export_stuff (vector<Sample*>& buffers, uint32_t nbufs, jack_nframes
 	return 0;
 }
 
-void
-AudioTrack::set_latency_delay (jack_nframes_t longest_session_latency)
-{
-	Route::set_latency_delay (longest_session_latency);
-	diskstream->set_roll_delay (_roll_delay);
-}
-
-jack_nframes_t
-AudioTrack::update_total_latency ()
-{
-	_own_latency = 0;
-
-	for (RedirectList::iterator i = _redirects.begin(); i != _redirects.end(); ++i) {
-		if ((*i)->active ()) {
-			_own_latency += (*i)->latency ();
-		}
-	}
-
-	set_port_latency (_own_latency);
-
-	return _own_latency;
-}
-
 void
 AudioTrack::bounce (InterThreadInfo& itt)
 {
-	vector<Source*> srcs;
-	_session.write_one_track (*this, 0, _session.current_end_frame(), false, srcs, itt);
+	vector<boost::shared_ptr<AudioSource> > srcs;
+	_session.write_one_audio_track (*this, 0, _session.current_end_frame(), false, srcs, itt);
 }
 
 
 void
-AudioTrack::bounce_range (jack_nframes_t start, jack_nframes_t end, InterThreadInfo& itt)
+AudioTrack::bounce_range (nframes_t start, nframes_t end, InterThreadInfo& itt)
 {
-	vector<Source*> srcs;
-	_session.write_one_track (*this, start, end, false, srcs, itt);
+	vector<boost::shared_ptr<AudioSource> > srcs;
+	_session.write_one_audio_track (*this, start, end, false, srcs, itt);
 }
 
 void
 AudioTrack::freeze (InterThreadInfo& itt)
 {
-	Insert* insert;
-	vector<Source*> srcs;
+	vector<boost::shared_ptr<AudioSource> > srcs;
 	string new_playlist_name;
-	Playlist* new_playlist;
+	boost::shared_ptr<Playlist> new_playlist;
 	string dir;
-	AudioRegion* region;
 	string region_name;
+	boost::shared_ptr<AudioDiskstream> diskstream = audio_diskstream();
 	
-	if ((_freeze_record.playlist = diskstream->playlist()) == 0) {
+	if ((_freeze_record.playlist = boost::dynamic_pointer_cast<AudioPlaylist>(diskstream->playlist())) == 0) {
 		return;
 	}
 
 	uint32_t n = 1;
 
-	while (n < ULONG_MAX) {
-
+	while (n < (UINT_MAX-1)) {
+	 
 		string candidate;
 		
-		candidate = compose ("<F%2>%1", _freeze_record.playlist->name(), n);
+		candidate = string_compose ("<F%2>%1", _freeze_record.playlist->name(), n);
 
 		if (_session.playlist_by_name (candidate) == 0) {
 			new_playlist_name = candidate;
@@ -880,14 +804,14 @@ AudioTrack::freeze (InterThreadInfo& itt)
 
 	} 
 
-	if (n == ULONG_MAX) {
-		error << compose (_("There are too many frozen versions of playlist \"%1\""
-				    " to create another one"), _freeze_record.playlist->name())
-		      << endmsg;
+	if (n == (UINT_MAX-1)) {
+	  error << string_compose (X_("There are too many frozen versions of playlist \"%1\""
+	  		    " to create another one"), _freeze_record.playlist->name())
+	       << endmsg;
 		return;
 	}
 
-	if (_session.write_one_track (*this, 0, _session.current_end_frame(), true, srcs, itt)) {
+	if (_session.write_one_audio_track (*this, _session.current_start_frame(), _session.current_end_frame(), true, srcs, itt)) {
 		return;
 	}
 
@@ -895,18 +819,18 @@ AudioTrack::freeze (InterThreadInfo& itt)
 	_freeze_record.have_mementos = true;
 
 	{
-		RWLockMonitor lm (redirect_lock, false, __LINE__, __FILE__);
+		Glib::RWLock::ReaderLock lm (redirect_lock);
 		
 		for (RedirectList::iterator r = _redirects.begin(); r != _redirects.end(); ++r) {
 			
-			if ((insert = dynamic_cast<Insert*>(*r)) != 0) {
+			boost::shared_ptr<Insert> insert;
+
+			if ((insert = boost::dynamic_pointer_cast<Insert>(*r)) != 0) {
 				
-				FreezeRecordInsertInfo* frii  = new FreezeRecordInsertInfo ((*r)->get_state());
+				FreezeRecordInsertInfo* frii  = new FreezeRecordInsertInfo ((*r)->get_state(), insert);
 				
-				frii->insert = insert;
 				frii->id = insert->id();
-				frii->memento = (*r)->get_memento();
-				
+
 				_freeze_record.insert_info.push_back (frii);
 				
 				/* now deactivate the insert */
@@ -916,23 +840,23 @@ AudioTrack::freeze (InterThreadInfo& itt)
 		}
 	}
 
-	new_playlist = new AudioPlaylist (_session, new_playlist_name, false);
+	new_playlist = PlaylistFactory::create (_session, new_playlist_name, false);
 	region_name = new_playlist_name;
 
 	/* create a new region from all filesources, keep it private */
 
-	region = new AudioRegion (srcs, 0, srcs[0]->length(), 
-				  region_name, 0, 
-				  (AudioRegion::Flag) (AudioRegion::WholeFile|AudioRegion::DefaultFlags),
-				  false);
+	boost::shared_ptr<Region> region (RegionFactory::create (srcs, 0, srcs[0]->length(), 
+								 region_name, 0, 
+								 (AudioRegion::Flag) (AudioRegion::WholeFile|AudioRegion::DefaultFlags),
+								 false));
 
 	new_playlist->set_orig_diskstream_id (diskstream->id());
-	new_playlist->add_region (*region, 0);
+	new_playlist->add_region (region, _session.current_start_frame());
 	new_playlist->set_frozen (true);
 	region->set_locked (true);
 
-	diskstream->use_playlist (dynamic_cast<AudioPlaylist*>(new_playlist));
-	diskstream->set_record_enabled (false, this);
+	diskstream->use_playlist (boost::dynamic_pointer_cast<AudioPlaylist>(new_playlist));
+	diskstream->set_record_enabled (false);
 
 	_freeze_record.state = Frozen;
 	FreezeChange(); /* EMIT SIGNAL */
@@ -942,7 +866,7 @@ void
 AudioTrack::unfreeze ()
 {
 	if (_freeze_record.playlist) {
-		diskstream->use_playlist (_freeze_record.playlist);
+		audio_diskstream()->use_playlist (_freeze_record.playlist);
 
 		if (_freeze_record.have_mementos) {
 
@@ -952,7 +876,7 @@ AudioTrack::unfreeze ()
 
 		} else {
 
-			RWLockMonitor lm (redirect_lock, false, __LINE__, __FILE__); // should this be a write lock? jlc
+			Glib::RWLock::ReaderLock lm (redirect_lock); // should this be a write lock? jlc
 			for (RedirectList::iterator i = _redirects.begin(); i != _redirects.end(); ++i) {
 				for (vector<FreezeRecordInsertInfo*>::iterator ii = _freeze_record.insert_info.begin(); ii != _freeze_record.insert_info.end(); ++ii) {
 					if ((*ii)->id == (*i)->id()) {
@@ -963,111 +887,10 @@ AudioTrack::unfreeze ()
 			}
 		}
 		
-		_freeze_record.playlist = 0;
+		_freeze_record.playlist.reset ();
 	}
 
 	_freeze_record.state = UnFrozen;
 	FreezeChange (); /* EMIT SIGNAL */
 }
 
-AudioTrack::FreezeRecord::~FreezeRecord ()
-{
-	for (vector<FreezeRecordInsertInfo*>::iterator i = insert_info.begin(); i != insert_info.end(); ++i) {
-		delete *i;
-	}
-}
-
-AudioTrack::FreezeState
-AudioTrack::freeze_state() const
-{
-	return _freeze_record.state;
-}
-
-
-void
-AudioTrack::reset_midi_control (MIDI::Port* port, bool on)
-{
-	MIDI::channel_t chn;
-	MIDI::eventType ev;
-	MIDI::byte extra;
-
-	Route::reset_midi_control (port, on);
-	
-	_midi_rec_enable_control.get_control_info (chn, ev, extra);
-	if (!on) {
-		chn = -1;
-	}
-	_midi_rec_enable_control.midi_rebind (port, chn);
-}
-
-void
-AudioTrack::send_all_midi_feedback ()
-{
-	if (_session.get_midi_feedback()) {
-
-		Route::send_all_midi_feedback();
-
-		_midi_rec_enable_control.send_feedback (record_enabled());
-	}
-}
-
-
-AudioTrack::MIDIRecEnableControl::MIDIRecEnableControl (AudioTrack& s,  MIDI::Port* port)
-	: MIDI::Controllable (port, 0), track (s), setting(false)
-{
-	last_written = false; /* XXX need a good out of bound value */
-}
-
-void
-AudioTrack::MIDIRecEnableControl::set_value (float val)
-{
-	bool bval = ((val >= 0.5f) ? true: false);
-	
-	setting = true;
-	track.set_record_enable (bval, this);
-	setting = false;
-}
-
-void
-AudioTrack::MIDIRecEnableControl::send_feedback (bool value)
-{
-
-	if (!setting && get_midi_feedback()) {
-		MIDI::byte val = (MIDI::byte) (value ? 127: 0);
-		MIDI::channel_t ch = 0;
-		MIDI::eventType ev = MIDI::none;
-		MIDI::byte additional = 0;
-		MIDI::EventTwoBytes data;
-	    
-		if (get_control_info (ch, ev, additional)) {
-			data.controller_number = additional;
-			data.value = val;
-
-			track._session.send_midi_message (get_port(), ev, ch, data);
-		}
-	}
-	
-}
-
-MIDI::byte*
-AudioTrack::MIDIRecEnableControl::write_feedback (MIDI::byte* buf, int32_t& bufsize, bool val, bool force)
-{
-	if (get_midi_feedback()) {
-
-		MIDI::channel_t ch = 0;
-		MIDI::eventType ev = MIDI::none;
-		MIDI::byte additional = 0;
-
-		if (get_control_info (ch, ev, additional)) {
-			if (val != last_written || force) {
-				*buf++ = ev & ch;
-				*buf++ = additional; /* controller number */
-				*buf++ = (MIDI::byte) (val ? 127: 0);
-				last_written = val;
-				bufsize -= 3;
-			}
-		}
-	}
-
-	return buf;
-}
