@@ -15,15 +15,18 @@
     along with this program; if not, write to the Free Software
     Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
 
-    $Id: audioengine.cc,v 1.77 2005/05/11 02:27:48 pauld Exp $
 */
 
 #include <unistd.h>
 #include <cerrno>
 #include <vector>
+
+#include <glibmm/timer.h>
 #include <pbd/pthread_utils.h>
+#include <pbd/stacktrace.h>
 
 #include <ardour/audioengine.h>
+#include <ardour/buffer.h>
 #include <ardour/port.h>
 #include <ardour/session.h>
 #include <ardour/cycle_timer.h>
@@ -38,13 +41,20 @@
 
 using namespace std;
 using namespace ARDOUR;
+using namespace PBD;
 
-jack_nframes_t Port::short_over_length = 2;
-jack_nframes_t Port::long_over_length = 10;
+gint AudioEngine::m_meter_exit;
+
+static void 
+ardour_jack_error (const char* msg) 
+{
+	// throw JACK errors away - they cause visual clutter
+	// error << "JACK: " << msg << endmsg;
+}
 
 AudioEngine::AudioEngine (string client_name) 
+	: ports (new Ports)
 {
-	pthread_cond_init (&session_removed, 0);
 	session = 0;
 	session_remove_pending = false;
 	_running = false;
@@ -60,16 +70,28 @@ AudioEngine::AudioEngine (string client_name)
 	_freewheeling = false;
 	_freewheel_thread_registered = false;
 
+	m_meter_thread = 0;
+	g_atomic_int_set (&m_meter_exit, 0);
+
 	if (connect_to_jack (client_name)) {
 		throw NoBackendAvailable ();
 	}
 
+	start_metering_thread();
+
 }
 
 AudioEngine::~AudioEngine ()
 {
-	if (_running) {
-		jack_client_close (_jack);
+	{
+		Glib::Mutex::Lock tm (_process_lock);
+		session_removed.signal ();
+		
+		if (_running) {
+			jack_client_close (_jack);
+		}
+		
+		stop_metering_thread ();
 	}
 }
 
@@ -80,13 +102,7 @@ _thread_init_callback (void *arg)
 	   knows about it.
 	*/
 
-	PBD::ThreadCreated (pthread_self(), X_("Audioengine"));
-
-#ifdef VST_SUPPORT
-	if (Config->get_use_vst()) {
-		fst_adopt_thread ();
-	}
-#endif
+	PBD::ThreadCreatedWithRequestSize (pthread_self(), X_("Audioengine"), 4096);
 }
 
 int
@@ -95,7 +111,7 @@ AudioEngine::start ()
 	if (!_running) {
 
 		if (session) {
-			jack_nframes_t blocksize = jack_get_buffer_size (_jack);
+			nframes_t blocksize = jack_get_buffer_size (_jack);
 
 			session->set_block_size (blocksize);
 			session->set_frame_rate (jack_get_sample_rate (_jack));
@@ -136,7 +152,7 @@ AudioEngine::start ()
 			_has_run = true;
 			Running(); /* EMIT SIGNAL */
 		} else {
-			error << _("cannot activate JACK client") << endmsg;
+			// error << _("cannot activate JACK client") << endmsg;
 		}
 	}
 
@@ -144,11 +160,18 @@ AudioEngine::start ()
 }
 
 int
-AudioEngine::stop ()
+AudioEngine::stop (bool forever)
 {
 	if (_running) {
 		_running = false;
-		jack_deactivate (_jack);
+		if (forever) {
+			jack_client_t* foo = _jack;
+			_jack = 0;
+			jack_client_close (foo);
+			stop_metering_thread ();
+		} else {
+			jack_deactivate (_jack);
+		}
 		Stopped(); /* EMIT SIGNAL */
 	}
 
@@ -156,20 +179,38 @@ AudioEngine::stop ()
 }
 
 
-void
-AudioEngine::_jack_timebase_callback (jack_transport_state_t state, jack_nframes_t nframes,
+bool
+AudioEngine::get_sync_offset (nframes_t& offset) const
+{
 
-									  jack_position_t* pos, int new_position, void *arg)
+#ifdef HAVE_JACK_VIDEO_SUPPORT
+
+	jack_position_t pos;
+	
+	(void) jack_transport_query (_jack, &pos);
+
+	if (pos.valid & JackVideoFrameOffset) {
+		offset = pos.video_offset;
+		return true;
+	}
+
+#endif
+
+	return false;
+}
+
+void
+AudioEngine::_jack_timebase_callback (jack_transport_state_t state, nframes_t nframes,
+				      jack_position_t* pos, int new_position, void *arg)
 {
 	static_cast<AudioEngine*> (arg)->jack_timebase_callback (state, nframes, pos, new_position);
 }
 
 void
-AudioEngine::jack_timebase_callback (jack_transport_state_t state, jack_nframes_t nframes,
-
-									 jack_position_t* pos, int new_position)
+AudioEngine::jack_timebase_callback (jack_transport_state_t state, nframes_t nframes,
+				     jack_position_t* pos, int new_position)
 {
-	if (session && session->synced_to_jack()) {
+	if (_jack && session && session->synced_to_jack()) {
 		session->jack_timebase_callback (state, nframes, pos, new_position);
 	}
 }
@@ -183,29 +224,35 @@ AudioEngine::_jack_sync_callback (jack_transport_state_t state, jack_position_t*
 int
 AudioEngine::jack_sync_callback (jack_transport_state_t state, jack_position_t* pos)
 {
-	if (session) {
+	if (_jack && session) {
 		return session->jack_sync_callback (state, pos);
-	} else {
-		return true;
 	}
+
+	return true;
 }
 
 int
 AudioEngine::_xrun_callback (void *arg)
 {
-	 static_cast<AudioEngine *>(arg)->Xrun (); /* EMIT SIGNAL */
+	AudioEngine* ae = static_cast<AudioEngine*> (arg);
+	if (ae->jack()) {
+		ae->Xrun (); /* EMIT SIGNAL */
+	}
 	return 0;
 }
 
 int
 AudioEngine::_graph_order_callback (void *arg)
 {
-	 static_cast<AudioEngine *>(arg)->GraphReordered (); /* EMIT SIGNAL */
+	AudioEngine* ae = static_cast<AudioEngine*> (arg);
+	if (ae->jack()) {
+		ae->GraphReordered (); /* EMIT SIGNAL */
+	}
 	return 0;
 }
 
 int
-AudioEngine::_process_callback (jack_nframes_t nframes, void *arg)
+AudioEngine::_process_callback (nframes_t nframes, void *arg)
 {
 	return static_cast<AudioEngine *> (arg)->process_callback (nframes);
 }
@@ -216,65 +263,13 @@ AudioEngine::_freewheel_callback (int onoff, void *arg)
 	static_cast<AudioEngine*>(arg)->_freewheeling = onoff;
 }
 
-void 
-AudioEngine::meter (Port *port, jack_nframes_t nframes)
-{
-	double peak;
-	uint32_t overlen;
-	jack_default_audio_sample_t *buf;
-	
-	buf = port->get_buffer (nframes);
-	peak = port->_peak;
-	overlen = port->overlen;
-	
-	{
-		for (jack_nframes_t n = 0; n < nframes; ++n) {
-			
-			/* 1) peak metering */
-			
-			peak = f_max (peak, buf[n]);
-			
-			/* 2) clip/over metering */
-			
-			if (buf[n] >= 1.0) {
-				overlen++;
-			} else if (overlen) {
-				if (overlen > Port::short_over_length) {
-					port->_short_overs++;
-				}
-				if (overlen > Port::long_over_length) {
-					port->_long_overs++;
-				}
-				overlen = 0;
-			}
-		}
-	}
-	
-        /* post-loop check on the final status of overlen */
-	
-	if (overlen > Port::short_over_length) {
-		port->_short_overs++;
-	}
-	if (overlen > Port::long_over_length) {
-		port->_short_overs++;
-	}
-
-	if (peak > 0.0) {
-		port->_peak_db= 20 * fast_log10 (peak);
-	} else {
-		port->_peak_db = minus_infinity();
-	}
-	
-	port->_peak = peak;
-	port->overlen = overlen;
-}
-
 int
-AudioEngine::process_callback (jack_nframes_t nframes)
+AudioEngine::process_callback (nframes_t nframes)
 {
-	TentativeLockMonitor tm (_process_lock, __LINE__, __FILE__);
-	jack_nframes_t next_processed_frames;
-
+	// CycleTimer ct ("AudioEngine::process");
+	Glib::Mutex::Lock tm (_process_lock, Glib::TRY_LOCK);
+	nframes_t next_processed_frames;
+	
 	/* handle wrap around of total frames counter */
 
 	if (max_frames - _processed_frames < nframes) {
@@ -291,27 +286,20 @@ AudioEngine::process_callback (jack_nframes_t nframes)
 	if (session_remove_pending) {
 		session = 0;
 		session_remove_pending = false;
-		pthread_cond_signal (&session_removed);
+		session_removed.signal();
 		_processed_frames = next_processed_frames;
 		return 0;
 	}
 
 	if (_freewheeling) {
 		if (Freewheel (nframes)) {
+			cerr << "Freewheeling returned non-zero!\n";
 			_freewheeling = false;
 			jack_set_freewheel (_jack, false);
 		}
 		return 0;
 	}
 
-	/* do input peak metering */
-
-	for (Ports::iterator i = ports.begin(); i != ports.end(); ++i) {
-		if ((*i)->metering) {
-			meter ((*i), nframes);
-		}
-	}
-	
 	session->process (nframes);
 
 	if (!_running) {
@@ -322,15 +310,18 @@ AudioEngine::process_callback (jack_nframes_t nframes)
 		_processed_frames = next_processed_frames;
 		return 0;
 	}
-		
+
 	if (last_monitor_check + monitor_check_interval < next_processed_frames) {
-		for (Ports::iterator i = ports.begin(); i != ports.end(); ++i) {
+
+		boost::shared_ptr<Ports> p = ports.reader();
+
+		for (Ports::iterator i = p->begin(); i != p->end(); ++i) {
 			
 			Port *port = (*i);
 			bool x;
 			
-			if (port->last_monitor != (x = port->monitoring_input ())) {
-				port->last_monitor = x;
+			if (port->_last_monitor != (x = port->monitoring_input ())) {
+				port->_last_monitor = x;
 				/* XXX I think this is dangerous, due to 
 				   a likely mutex in the signal handlers ...
 				*/
@@ -345,13 +336,13 @@ AudioEngine::process_callback (jack_nframes_t nframes)
 }
 
 int
-AudioEngine::_sample_rate_callback (jack_nframes_t nframes, void *arg)
+AudioEngine::_sample_rate_callback (nframes_t nframes, void *arg)
 {
 	return static_cast<AudioEngine *> (arg)->jack_sample_rate_callback (nframes);
 }
 
 int
-AudioEngine::jack_sample_rate_callback (jack_nframes_t nframes)
+AudioEngine::jack_sample_rate_callback (nframes_t nframes)
 {
 	_frame_rate = nframes;
 	_usecs_per_cycle = (int) floor ((((double) frames_per_cycle() / nframes)) * 1000000.0);
@@ -360,7 +351,7 @@ AudioEngine::jack_sample_rate_callback (jack_nframes_t nframes)
 
 	monitor_check_interval = nframes / 10;
 	last_monitor_check = 0;
-
+	
 	if (session) {
 		session->set_frame_rate (nframes);
 	}
@@ -371,19 +362,21 @@ AudioEngine::jack_sample_rate_callback (jack_nframes_t nframes)
 }
 
 int
-AudioEngine::_bufsize_callback (jack_nframes_t nframes, void *arg)
+AudioEngine::_bufsize_callback (nframes_t nframes, void *arg)
 {
 	return static_cast<AudioEngine *> (arg)->jack_bufsize_callback (nframes);
 }
 
 int
-AudioEngine::jack_bufsize_callback (jack_nframes_t nframes)
+AudioEngine::jack_bufsize_callback (nframes_t nframes)
 {
 	_buffer_size = nframes;
 	_usecs_per_cycle = (int) floor ((((double) nframes / frame_rate())) * 1000000.0);
 	last_monitor_check = 0;
 
-	for (Ports::iterator i = ports.begin(); i != ports.end(); ++i) {
+	boost::shared_ptr<Ports> p = ports.reader();
+
+	for (Ports::iterator i = p->begin(); i != p->end(); ++i) {
 		(*i)->reset();
 	}
 
@@ -394,63 +387,110 @@ AudioEngine::jack_bufsize_callback (jack_nframes_t nframes)
 	return 0;
 }
 
+void
+AudioEngine::stop_metering_thread ()
+{
+	if (m_meter_thread) {
+		g_atomic_int_set (&m_meter_exit, 1);
+		m_meter_thread->join ();
+		m_meter_thread = 0;
+	}
+}
+
+void
+AudioEngine::start_metering_thread ()
+{
+	if (m_meter_thread == 0) {
+		m_meter_thread = Glib::Thread::create (sigc::mem_fun(this, &AudioEngine::meter_thread),
+				500000, true, true, Glib::THREAD_PRIORITY_NORMAL);
+	}
+}
+
+void
+AudioEngine::meter_thread ()
+{
+	while (true) {
+		Glib::usleep (10000); /* 1/100th sec interval */
+		if (g_atomic_int_get(&m_meter_exit)) {
+			break;
+		}
+		IO::update_meters ();
+	}
+}
+
 void 
 AudioEngine::set_session (Session *s)
 {
+	Glib::Mutex::Lock pl (_process_lock);
+
 	if (!session) {
-		s->set_block_size (jack_get_buffer_size (_jack));
-		s->set_frame_rate (jack_get_sample_rate (_jack));
+
 		session = s;
+
+		nframes_t blocksize = jack_get_buffer_size (_jack);
+		
+		/* page in as much of the session process code as we
+		   can before we really start running.
+		*/
+		
+		session->process (blocksize);
+		session->process (blocksize);
+		session->process (blocksize);
+		session->process (blocksize);
+		session->process (blocksize);
+		session->process (blocksize);
+		session->process (blocksize);
+		session->process (blocksize);
 	}
 }
 
 void 
 AudioEngine::remove_session ()
 {
-	LockMonitor lm (_process_lock, __LINE__, __FILE__);
+	Glib::Mutex::Lock lm (_process_lock);
 
 	if (_running) {
 
 		if (session) {
 			session_remove_pending = true;
-			pthread_cond_wait (&session_removed, _process_lock.mutex());
-		} 
+			session_removed.wait(_process_lock);
+		}
 
 	} else {
-
 		session = 0;
-
 	}
 	
 	remove_all_ports ();
-
 }
 
 Port *
-AudioEngine::register_audio_input_port (const string& portname)
+AudioEngine::register_input_port (DataType type, const string& portname)
 {
 	if (!_running) {
 		if (!_has_run) {
-			fatal << _("register audio input port called before engine was started") << endmsg;
+			fatal << _("register input port called before engine was started") << endmsg;
 			/*NOTREACHED*/
 		} else {
 			return 0;
 		}
 	}
 
-	jack_port_t *p = jack_port_register (_jack, portname.c_str(), JACK_DEFAULT_AUDIO_TYPE, JackPortIsInput, 0);
+	jack_port_t *p = jack_port_register (_jack, portname.c_str(), type.to_jack_type(), JackPortIsInput, 0);
 
 	if (p) {
 
 		Port *newport;
+
 		if ((newport = new Port (p)) != 0) {
-			ports.insert (ports.begin(), newport);
+			RCUWriter<Ports> writer (ports);
+			boost::shared_ptr<Ports> ps = writer.get_copy ();
+			ps->insert (ps->begin(), newport);
+			/* writer goes out of scope, forces update */
 		}
+
 		return newport;
 
 	} else {
-
-		pthread_mutex_unlock (_process_lock.mutex());
 		throw PortRegistrationFailure();
 	}
 
@@ -458,11 +498,11 @@ AudioEngine::register_audio_input_port (const string& portname)
 }
 
 Port *
-AudioEngine::register_audio_output_port (const string& portname)
+AudioEngine::register_output_port (DataType type, const string& portname)
 {
 	if (!_running) {
 		if (!_has_run) {
-			fatal << _("register audio output port called before engine was started") << endmsg;
+			fatal << _("register output port called before engine was started") << endmsg;
 			/*NOTREACHED*/
 		} else {
 			return 0;
@@ -471,20 +511,31 @@ AudioEngine::register_audio_output_port (const string& portname)
 
 	jack_port_t *p;
 
-	if ((p = jack_port_register (_jack, portname.c_str(), JACK_DEFAULT_AUDIO_TYPE, JackPortIsOutput, 0)) != 0) {
-		Port *newport = new Port (p);
-		ports.insert (ports.begin(), newport);
+	if ((p = jack_port_register (_jack, portname.c_str(),
+		type.to_jack_type(), JackPortIsOutput, 0)) != 0) {
+
+		Port *newport = 0;
+
+		{
+			RCUWriter<Ports> writer (ports);
+			boost::shared_ptr<Ports> ps = writer.get_copy ();
+			
+			newport = new Port (p);
+			ps->insert (ps->begin(), newport);
+
+			/* writer goes out of scope, forces update */
+		}
+
 		return newport;
 
 	} else {
-
-		pthread_mutex_unlock (_process_lock.mutex());
 		throw PortRegistrationFailure ();
 	}
 
 	return 0;
 }
 
+
 int          
 AudioEngine::unregister_port (Port *port)
 {
@@ -497,15 +548,23 @@ AudioEngine::unregister_port (Port *port)
 
 	if (port) {
 
-		int ret = jack_port_unregister (_jack, port->port);
+		int ret = jack_port_unregister (_jack, port->_port);
 		
 		if (ret == 0) {
+			
+			{
 
-			for (Ports::iterator i = ports.begin(); i != ports.end(); ++i) {
-				if ((*i) == port) {
-					ports.erase (i);
-					break;
+				RCUWriter<Ports> writer (ports);
+				boost::shared_ptr<Ports> ps = writer.get_copy ();
+				
+				for (Ports::iterator i = ps->begin(); i != ps->end(); ++i) {
+					if ((*i) == port) {
+						ps->erase (i);
+						break;
+					}
 				}
+
+				/* writer goes out of scope, forces update */
 			}
 
 			remove_connections_for (port);
@@ -538,8 +597,12 @@ AudioEngine::connect (const string& source, const string& destination)
 	if (ret == 0) {
 		pair<string,string> c (s, d);
 		port_connections.push_back (c);
+	} else if (ret == EEXIST) {
+		error << string_compose(_("AudioEngine: connection already exists: %1 (%2) to %3 (%4)"), 
+				 source, s, destination, d) 
+		      << endmsg;
 	} else {
-		error << compose(_("AudioEngine: cannot connect %1 (%2) to %3 (%4)"), 
+		error << string_compose(_("AudioEngine: cannot connect %1 (%2) to %3 (%4)"), 
 				 source, s, destination, d) 
 		      << endmsg;
 	}
@@ -588,7 +651,7 @@ AudioEngine::disconnect (Port *port)
 		}
 	}
 
-	int ret = jack_port_disconnect (_jack, port->port);
+	int ret = jack_port_disconnect (_jack, port->_port);
 
 	if (ret == 0) {
 		remove_connections_for (port);
@@ -598,7 +661,7 @@ AudioEngine::disconnect (Port *port)
 
 }
 
-jack_nframes_t
+nframes_t
 AudioEngine::frame_rate ()
 {
 	if (_jack) {
@@ -615,7 +678,7 @@ AudioEngine::frame_rate ()
 	}
 }
 
-jack_nframes_t
+nframes_t
 AudioEngine::frames_per_cycle ()
 {
 	if (_jack) {
@@ -635,7 +698,7 @@ AudioEngine::frames_per_cycle ()
 Port *
 AudioEngine::get_port_by_name (const string& portname, bool keep)
 {
-	LockMonitor lm (_process_lock, __LINE__, __FILE__);
+	Glib::Mutex::Lock lm (_process_lock);
 
 	if (!_running) {
 		if (!_has_run) {
@@ -648,7 +711,9 @@ AudioEngine::get_port_by_name (const string& portname, bool keep)
 	
 	/* check to see if we have a Port for this name already */
 
-	for (Ports::iterator i = ports.begin(); i != ports.end(); ++i) {
+	boost::shared_ptr<Ports> pr = ports.reader();
+	
+	for (Ports::iterator i = pr->begin(); i != pr->end(); ++i) {
 		if (portname == (*i)->name()) {
 			return (*i);
 		}
@@ -658,11 +723,20 @@ AudioEngine::get_port_by_name (const string& portname, bool keep)
 
 	if ((p = jack_port_by_name (_jack, portname.c_str())) != 0) {
 		Port *newport = new Port (p);
-		if (keep && newport->is_mine (_jack)) {
-			ports.insert (newport);
+
+		{
+			if (keep && newport->is_mine (_jack)) {
+				RCUWriter<Ports> writer (ports);
+				boost::shared_ptr<Ports> ps = writer.get_copy ();
+				ps->insert (newport);
+				/* writer goes out of scope, forces update */
+			}
 		}
+
 		return newport;
+
 	} else {
+
 		return 0;
 	}
 }
@@ -684,19 +758,18 @@ AudioEngine::get_ports (const string& port_name_pattern, const string& type_name
 void
 AudioEngine::halted (void *arg)
 {
-	AudioEngine *ae = reinterpret_cast<AudioEngine *> (arg);
+	AudioEngine* ae = static_cast<AudioEngine *> (arg);
 
 	ae->_running = false;
-	ae->_jack = 0;
-
 	ae->_buffer_size = 0;
 	ae->_frame_rate = 0;
+	ae->_jack = 0;
 
 	ae->Halted(); /* EMIT SIGNAL */
 }
 
 uint32_t
-AudioEngine::n_physical_outputs () const
+AudioEngine::n_physical_audio_outputs () const
 {
 	const char ** ports;
 	uint32_t i = 0;
@@ -705,7 +778,7 @@ AudioEngine::n_physical_outputs () const
 		return 0;
 	}
 
-	if ((ports = jack_get_ports (_jack, NULL, NULL, JackPortIsPhysical|JackPortIsInput)) == 0) {
+	if ((ports = jack_get_ports (_jack, NULL, JACK_DEFAULT_AUDIO_TYPE, JackPortIsPhysical|JackPortIsInput)) == 0) {
 		return 0;
 	}
 
@@ -717,7 +790,7 @@ AudioEngine::n_physical_outputs () const
 }
 
 uint32_t
-AudioEngine::n_physical_inputs () const
+AudioEngine::n_physical_audio_inputs () const
 {
 	const char ** ports;
 	uint32_t i = 0;
@@ -726,7 +799,7 @@ AudioEngine::n_physical_inputs () const
 		return 0;
 	}
 	
-	if ((ports = jack_get_ports (_jack, NULL, NULL, JackPortIsPhysical|JackPortIsOutput)) == 0) {
+	if ((ports = jack_get_ports (_jack, NULL, JACK_DEFAULT_AUDIO_TYPE, JackPortIsPhysical|JackPortIsOutput)) == 0) {
 		return 0;
 	}
 
@@ -737,9 +810,52 @@ AudioEngine::n_physical_inputs () const
 	return i;
 }
 
-string
+void
+AudioEngine::get_physical_audio_inputs (vector<string>& ins)
+{
+	const char ** ports;
+	uint32_t i = 0;
+	
+	if (!_jack) {
+		return;
+	}
+	
+	if ((ports = jack_get_ports (_jack, NULL, JACK_DEFAULT_AUDIO_TYPE, JackPortIsPhysical|JackPortIsOutput)) == 0) {
+		return;
+	}
 
-AudioEngine::get_nth_physical (uint32_t n, int flag)
+	if (ports) {
+		for (i = 0; ports[i]; ++i) {
+			ins.push_back (ports[i]);
+		}
+		free (ports);
+	}
+}
+
+void
+AudioEngine::get_physical_audio_outputs (vector<string>& outs)
+{
+	const char ** ports;
+	uint32_t i = 0;
+	
+	if (!_jack) {
+		return;
+	}
+	
+	if ((ports = jack_get_ports (_jack, NULL, JACK_DEFAULT_AUDIO_TYPE, JackPortIsPhysical|JackPortIsInput)) == 0) {
+		return;
+	}
+
+	if (ports) {
+		for (i = 0; ports[i]; ++i) {
+			outs.push_back (ports[i]);
+		}
+		free (ports);
+	}
+}
+
+string
+AudioEngine::get_nth_physical_audio (uint32_t n, int flag)
 {
 	const char ** ports;
 	uint32_t i;
@@ -754,7 +870,7 @@ AudioEngine::get_nth_physical (uint32_t n, int flag)
 		}
 	}
 
-	ports = jack_get_ports (_jack, NULL, NULL, JackPortIsPhysical|flag);
+	ports = jack_get_ports (_jack, NULL, JACK_DEFAULT_AUDIO_TYPE, JackPortIsPhysical|flag);
 	
 	if (ports == 0) {
 		return "";
@@ -771,7 +887,7 @@ AudioEngine::get_nth_physical (uint32_t n, int flag)
 	return ret;
 }
 
-jack_nframes_t
+nframes_t
 AudioEngine::get_port_total_latency (const Port& port)
 {
 	if (!_jack) {
@@ -786,7 +902,7 @@ AudioEngine::get_port_total_latency (const Port& port)
 		} 
 	}
 
-	return jack_port_get_total_latency (_jack, port.port);
+	return jack_port_get_total_latency (_jack, port._port);
 }
 
 void
@@ -808,7 +924,7 @@ AudioEngine::transport_start ()
 }
 
 void
-AudioEngine::transport_locate (jack_nframes_t where)
+AudioEngine::transport_locate (nframes_t where)
 {
 	// cerr << "tell JACK to locate to " << where << endl;
 	if (_jack) {
@@ -863,12 +979,19 @@ AudioEngine::remove_all_ports ()
 	/* process lock MUST be held */
 
 	if (_jack) {
-		for (Ports::iterator i = ports.begin(); i != ports.end(); ++i) {
-			jack_port_unregister (_jack, (*i)->port);
+		boost::shared_ptr<Ports> p = ports.reader();
+
+		for (Ports::iterator i = p->begin(); i != p->end(); ++i) {
+			jack_port_unregister (_jack, (*i)->_port);
 		}
 	}
 
-	ports.clear ();
+	{
+		RCUWriter<Ports> writer (ports);
+		boost::shared_ptr<Ports> ps = writer.get_copy ();
+		ps->clear ();
+	}
+
 	port_connections.clear ();
 }
 
@@ -903,22 +1026,15 @@ AudioEngine::connect_to_jack (string client_name)
 	_jack = jack_client_open (jack_client_name.c_str(), options, &status, server_name);
 	
 	if (_jack == NULL) {
-
-		if (status & JackServerFailed) {
-			error << _("Unable to connect to JACK server") << endmsg;
-		}
-		
-		error << compose (_("Could not connect to JACK server as  \"%1\""), jack_client_name) <<  endmsg;
+		/* just return without an error message. something else will take care of it */
 		return -1;
 	}
 
-	if (status & JackServerStarted) {
-		info << _("JACK server started") << endmsg;
-	}
-
 	if (status & JackNameNotUnique) {
 		jack_client_name = jack_get_client_name (_jack);
 	}
+
+	jack_set_error_function (ardour_jack_error);
 	
 	return 0;
 }
@@ -946,9 +1062,7 @@ AudioEngine::disconnect_from_jack ()
 		return 0;
 	}
 
-	if (jack_client_close (_jack)) {
-		error << _("cannot shutdown connection to JACK") << endmsg;
-	}
+	jack_client_close (_jack);
 
 	_buffer_size = 0;
 	_frame_rate = 0;
@@ -968,7 +1082,7 @@ AudioEngine::reconnect_to_jack ()
 	if (_jack) {
 		disconnect_from_jack ();
 		/* XXX give jackd a chance */
-		usleep (250000);
+		Glib::usleep (250000);
 	}
 
 	if (connect_to_jack (jack_client_name)) {
@@ -978,7 +1092,9 @@ AudioEngine::reconnect_to_jack ()
 
 	Ports::iterator i;
 
-	for (i = ports.begin(); i != ports.end(); ++i) {
+	boost::shared_ptr<Ports> p = ports.reader ();
+
+	for (i = p->begin(); i != p->end(); ++i) {
 
 		/* XXX hack hack hack */
 
@@ -987,8 +1103,8 @@ AudioEngine::reconnect_to_jack ()
 		
 		short_name = long_name.substr (long_name.find_last_of (':') + 1);
 
-		if (((*i)->port = jack_port_register (_jack, short_name.c_str(), (*i)->type(), (*i)->flags(), 0)) == 0) {
-			error << compose (_("could not reregister %1"), (*i)->name()) << endmsg;
+		if (((*i)->_port = jack_port_register (_jack, short_name.c_str(), (*i)->type(), (*i)->flags(), 0)) == 0) {
+			error << string_compose (_("could not reregister %1"), (*i)->name()) << endmsg;
 			break;
 		} else {
 		}
@@ -1000,16 +1116,18 @@ AudioEngine::reconnect_to_jack ()
 		}
 	}
 
-	if (i != ports.end()) {
-		for (Ports::iterator i = ports.begin(); i != ports.end(); ++i) {
-			jack_port_unregister (_jack, (*i)->port);
+	if (i != p->end()) {
+		/* failed */
+		for (Ports::iterator i = p->begin(); i != p->end(); ++i) {
+			jack_port_unregister (_jack, (*i)->_port);
 		}
 		return -1;
 	} 
 
 
 	if (session) {
-		jack_nframes_t blocksize = jack_get_buffer_size (_jack);
+		session->reset_jack_connection (_jack);
+		nframes_t blocksize = jack_get_buffer_size (_jack);
 		session->set_block_size (blocksize);
 		session->set_frame_rate (jack_get_sample_rate (_jack));
 	}
@@ -1045,7 +1163,7 @@ AudioEngine::reconnect_to_jack ()
 		
 		if ((err = jack_connect (_jack, (*i).first.c_str(), (*i).second.c_str())) != 0) {
 			if (err != EEXIST) {
-				error << compose (_("could not reconnect %1 and %2 (err = %3)"),
+				error << string_compose (_("could not reconnect %1 and %2 (err = %3)"),
 						  (*i).first, (*i).second, err)
 				      << endmsg;
 			}
@@ -1058,11 +1176,16 @@ AudioEngine::reconnect_to_jack ()
 }
 
 int
-AudioEngine::request_buffer_size (jack_nframes_t nframes)
+AudioEngine::request_buffer_size (nframes_t nframes)
 {
 	if (_jack) {
-		int ret = jack_set_buffer_size (_jack, nframes);
-		return ret;
+
+		if (nframes == jack_get_buffer_size (_jack)) {
+			return 0;
+		}
+
+		return jack_set_buffer_size (_jack, nframes);
+
 	} else {
 		return -1;
 	}
@@ -1113,3 +1236,12 @@ AudioEngine::make_port_name_non_relative (string portname)
 	return str;
 }
 
+bool
+AudioEngine::is_realtime () const
+{
+	if (_jack) {
+		return jack_is_realtime (_jack);
+	} else {
+		return false;
+	}
+}
