@@ -49,7 +49,7 @@ using namespace ARDOUR;
 using namespace PBD;
 
 uint32_t Route::order_key_cnt = 0;
-
+sigc::signal<void> Route::SyncOrderKeys;
 
 Route::Route (Session& sess, string name, int input_min, int input_max, int output_min, int output_max, Flag flg, DataType default_type)
 	: IO (sess, name, input_min, input_max, output_min, output_max, default_type),
@@ -157,9 +157,34 @@ void
 Route::set_order_key (const char* name, long n)
 {
 	order_keys[strdup(name)] = n;
+
+	if (Config->get_sync_all_route_ordering()) {
+		for (OrderKeys::iterator x = order_keys.begin(); x != order_keys.end(); ++x) {
+			x->second = n;
+		}
+	} 
+
 	_session.set_dirty ();
 }
 
+void
+Route::sync_order_keys ()
+{
+	uint32_t key;
+	
+	if (order_keys.empty()) {
+		return;
+	}
+	
+	OrderKeys::iterator x = order_keys.begin();
+	key = x->second;
+	++x;
+
+	for (; x != order_keys.end(); ++x) {
+		x->second = key;
+	}
+}
+
 void
 Route::inc_gain (gain_t fraction, void *src)
 {
@@ -454,13 +479,9 @@ Route::process_output_buffers (vector<Sample*>& bufs, uint32_t nbufs,
 		
 	    // OR recording 
 		
-		// h/w monitoring not in use 
+		// AND software monitoring required
 		
-		(!Config->get_monitoring_model() == HardwareMonitoring && 
-
-		 // AND software monitoring required
-
-		 Config->get_monitoring_model() == SoftwareMonitoring)) { 
+		(Config->get_monitoring_model() == SoftwareMonitoring)) {
 		
 		if (apply_gain_automation) {
 			
@@ -686,7 +707,6 @@ Route::process_output_buffers (vector<Sample*>& bufs, uint32_t nbufs,
 	   -------------------------------------------------------------------------------------------------- */
 
 	if (meter && (_meter_point == MeterPostFader)) {
-//		cerr << "meter post" << endl;
 
 		if ((_gain == 0 && !apply_gain_automation) || dmg == 0) {
 			uint32_t no = n_outputs();
