@@ -15,7 +15,6 @@
     along with this program; if not, write to the Free Software
     Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
 
-    $Id: session_transport.cc,v 1.104 2006/03/29 20:41:24 essej Exp $
 */
 
 #include <cmath>
@@ -23,12 +22,13 @@
 #include <unistd.h>
 
 #include <sigc++/bind.h>
-#include <sigc++/rettype.h>
+#include <sigc++/retype.h>
 
 #include <pbd/undo.h>
 #include <pbd/error.h>
-#include <pbd/lockmonitor.h>
+#include <glibmm/thread.h>
 #include <pbd/pthread_utils.h>
+#include <pbd/memento_command.h>
 
 #include <midi++/mmc.h>
 #include <midi++/port.h>
@@ -36,7 +36,7 @@
 #include <ardour/ardour.h>
 #include <ardour/audioengine.h>
 #include <ardour/session.h>
-#include <ardour/diskstream.h>
+#include <ardour/audio_diskstream.h>
 #include <ardour/auditioner.h>
 #include <ardour/slave.h>
 #include <ardour/location.h>
@@ -45,26 +45,28 @@
 
 using namespace std;
 using namespace ARDOUR;
-using namespace SigC;
+using namespace sigc;
+using namespace PBD;
 
 void
 Session::request_input_change_handling ()
 {
-	Event* ev = new Event (Event::InputConfigurationChange, Event::Add, Event::Immediate, 0, 0.0);
-	queue_event (ev);
+	if (!(_state_of_the_state & (InitialConnecting|Deletion))) {
+		Event* ev = new Event (Event::InputConfigurationChange, Event::Add, Event::Immediate, 0, 0.0);
+		queue_event (ev);
+	}
 }
 
 void
-Session::request_slave_source (SlaveSource src, jack_nframes_t pos)
+Session::request_slave_source (SlaveSource src)
 {
-	Event* ev = new Event (Event::SetSlaveSource, Event::Add, Event::Immediate, pos, 0.0);
+	Event* ev = new Event (Event::SetSlaveSource, Event::Add, Event::Immediate, 0, 0.0);
 
-	if (src == Session::JACK) {
-	  /* could set_seamless_loop() be disposed of entirely?*/
-	  set_seamless_loop (false);
+	if (src == JACK) {
+		/* could set_seamless_loop() be disposed of entirely?*/
+		Config->set_seamless_loop (false);
 	} else {
-
-	  set_seamless_loop (true);
+		Config->set_seamless_loop (true);
 	}
 	ev->slave = src;
 	queue_event (ev);
@@ -78,7 +80,7 @@ Session::request_transport_speed (float speed)
 }
 
 void
-Session::request_diskstream_speed (DiskStream& ds, float speed)
+Session::request_diskstream_speed (Diskstream& ds, float speed)
 {
 	Event* ev = new Event (Event::SetDiskstreamSpeed, Event::Add, Event::Immediate, 0, speed);
 	ev->set_ptr (&ds);
@@ -93,21 +95,21 @@ Session::request_stop (bool abort)
 }
 
 void
-Session::request_locate (jack_nframes_t target_frame, bool with_roll)
+Session::request_locate (nframes_t target_frame, bool with_roll)
 {
 	Event *ev = new Event (with_roll ? Event::LocateRoll : Event::Locate, Event::Add, Event::Immediate, target_frame, 0, false);
 	queue_event (ev);
 }
 
 void
-Session::force_locate (jack_nframes_t target_frame, bool with_roll)
+Session::force_locate (nframes_t target_frame, bool with_roll)
 {
 	Event *ev = new Event (with_roll ? Event::LocateRoll : Event::Locate, Event::Add, Event::Immediate, target_frame, 0, true);
 	queue_event (ev);
 }
 
 void
-Session::request_auto_loop (bool yn)
+Session::request_play_loop (bool yn)
 {
 	Event* ev;	
 	Location *location = _locations.auto_loop_location();
@@ -121,25 +123,10 @@ Session::request_auto_loop (bool yn)
 	ev = new Event (Event::SetLoop, Event::Add, Event::Immediate, 0, 0.0, yn);
 	queue_event (ev);
 
-	if (!yn && seamless_loop && transport_rolling()) {
+	if (!yn && Config->get_seamless_loop() && transport_rolling()) {
 		// request an immediate locate to refresh the diskstreams
 		// after disabling looping
-		request_locate (_transport_frame-1, true);
-	}
-}
-
-void
-Session::set_seamless_loop (bool yn)
-{
-	if (seamless_loop != yn) {
-		seamless_loop = yn;
-
-		if (auto_loop && transport_rolling()) {
-			// to reset diskstreams etc
-			request_auto_loop (true);
-		}
-		
-		ControlChanged (SeamlessLoop); /* EMIT */
+		request_locate (_transport_frame-1, false);
 	}
 }
 
@@ -165,22 +152,11 @@ Session::realtime_stop (bool abort)
 		decrement_transport_position (_worst_output_latency);
 #endif
 
-		if (_transport_frame > current_end_frame()) {
-			
-			/* first capture resets end location; later captures can only extend the length */
+		/* the duration change is not guaranteed to have happened, but is likely */
 
-			if (_end_location_is_free) {
-				end_location->set_end (_transport_frame);
-				_end_location_is_free = false;
-			} else if (_transport_frame > end_location->start()) {
-				end_location->set_end (_transport_frame);
-			}
-
-			post_transport_work = PostTransportWork (post_transport_work | PostTransportDuration);
-		}
+		post_transport_work = PostTransportWork (post_transport_work | PostTransportDuration);
 	}
 
-
 	if (abort) {
 		post_transport_work = PostTransportWork (post_transport_work | PostTransportAbort);
 	}
@@ -195,23 +171,32 @@ Session::realtime_stop (bool abort)
 		
 	_transport_speed = 0;
 
-	transport_sub_state = (auto_return ? AutoReturning : 0);
+	if (Config->get_use_video_sync()) {
+		waiting_for_sync_offset = true;
+	}
+
+	transport_sub_state = ((Config->get_slave_source() == None && Config->get_auto_return()) ? AutoReturning : 0);
 }
 
 void
 Session::butler_transport_work ()
 {
-	RWLockMonitor rm (route_lock, false, __LINE__, __FILE__);
-	RWLockMonitor dsm (diskstream_lock, false, __LINE__, __FILE__);
-		
+  restart:
+	bool finished;
+	boost::shared_ptr<RouteList> r = routes.reader ();
+	boost::shared_ptr<DiskstreamList> dsl = diskstreams.reader();
+
+	int on_entry = g_atomic_int_get (&butler_should_do_transport_work);
+	finished = true;
+
 	if (post_transport_work & PostTransportCurveRealloc) {
-		for (RouteList::iterator i = routes.begin(); i != routes.end(); ++i) {
+		for (RouteList::iterator i = r->begin(); i != r->end(); ++i) {
 			(*i)->curve_reallocate();
 		}
 	}
 
 	if (post_transport_work & PostTransportInputChange) {
-		for (DiskStreamList::iterator i = diskstreams.begin(); i != diskstreams.end(); ++i) {
+		for (DiskstreamList::iterator i = dsl->begin(); i != dsl->end(); ++i) {
 			(*i)->non_realtime_input_change ();
 		}
 	}
@@ -227,65 +212,91 @@ Session::butler_transport_work ()
 		cumulative_rf_motion = 0;
 		reset_rf_scale (0);
 
-		for (DiskStreamList::iterator i = diskstreams.begin(); i != diskstreams.end(); ++i) {
-			if (!(*i)->hidden()) {
-				if ((*i)->speed() != 1.0f || (*i)->speed() != -1.0f) {
-					(*i)->seek ((jack_nframes_t) (_transport_frame * (double) (*i)->speed()));
+		/* don't seek if locate will take care of that in non_realtime_stop() */
+
+		if (!(post_transport_work & PostTransportLocate)) {
+			
+			for (DiskstreamList::iterator i = dsl->begin(); i != dsl->end(); ++i) {
+				if (!(*i)->hidden()) {
+					if ((*i)->speed() != 1.0f || (*i)->speed() != -1.0f) {
+						(*i)->seek ((nframes_t) (_transport_frame * (double) (*i)->speed()));
+					}
+					else {
+						(*i)->seek (_transport_frame);
+					}
 				}
-				else {
-					(*i)->seek (_transport_frame);
+				if (on_entry != g_atomic_int_get (&butler_should_do_transport_work)) {
+					/* new request, stop seeking, and start again */
+					g_atomic_int_dec_and_test (&butler_should_do_transport_work);
+					goto restart;
 				}
 			}
 		}
 	}
 
 	if (post_transport_work & (PostTransportStop|PostTransportLocate)) {
-		non_realtime_stop (post_transport_work & PostTransportAbort);
+		non_realtime_stop (post_transport_work & PostTransportAbort, on_entry, finished);
+		if (!finished) {
+			g_atomic_int_dec_and_test (&butler_should_do_transport_work);
+			goto restart;
+		}
 	}
 
 	if (post_transport_work & PostTransportOverWrite) {
-		non_realtime_overwrite ();
+		non_realtime_overwrite (on_entry, finished);
+		if (!finished) {
+			g_atomic_int_dec_and_test (&butler_should_do_transport_work);
+			goto restart;
+		}
 	}
 
 	if (post_transport_work & PostTransportAudition) {
 		non_realtime_set_audition ();
 	}
-
-	atomic_dec (&butler_should_do_transport_work);
+	
+	g_atomic_int_dec_and_test (&butler_should_do_transport_work);
 }
 
 void
 Session::non_realtime_set_speed ()
 {
-	RWLockMonitor lm (diskstream_lock, false, __LINE__, __FILE__);
+	boost::shared_ptr<DiskstreamList> dsl = diskstreams.reader();
 
-	for (DiskStreamList::iterator i = diskstreams.begin(); i != diskstreams.end(); ++i) {
+	for (DiskstreamList::iterator i = dsl->begin(); i != dsl->end(); ++i) {
 		(*i)->non_realtime_set_speed ();
 	}
 }
 
 void
-Session::non_realtime_overwrite ()
+Session::non_realtime_overwrite (int on_entry, bool& finished)
 {
-	RWLockMonitor lm (diskstream_lock, false, __LINE__, __FILE__);
+	boost::shared_ptr<DiskstreamList> dsl = diskstreams.reader();
 
-	for (DiskStreamList::iterator i = diskstreams.begin(); i != diskstreams.end(); ++i) {
+	for (DiskstreamList::iterator i = dsl->begin(); i != dsl->end(); ++i) {
 		if ((*i)->pending_overwrite) {
 			(*i)->overwrite_existing_buffers ();
 		}
+		if (on_entry != g_atomic_int_get (&butler_should_do_transport_work)) {
+			finished = false;
+			return;
+		}
 	}
 }
 
 void
-Session::non_realtime_stop (bool abort)
+Session::non_realtime_stop (bool abort, int on_entry, bool& finished)
 {
 	struct tm* now;
 	time_t     xnow;
 	bool       did_record;
+	bool       saved;
 
 	did_record = false;
+	saved = false;
 
-	for (DiskStreamList::iterator i = diskstreams.begin(); i != diskstreams.end(); ++i) {
+	boost::shared_ptr<DiskstreamList> dsl = diskstreams.reader();
+	
+	for (DiskstreamList::iterator i = dsl->begin(); i != dsl->end(); ++i) {
 		if ((*i)->get_captured_frames () != 0) {
 			did_record = true;
 			break;
@@ -307,53 +318,71 @@ Session::non_realtime_stop (bool abort)
 
 	if (did_record) {
 		begin_reversible_command ("capture");
-
+		
 		Location* loc = _locations.end_location();
+		bool change_end = false;
+		
+		if (_transport_frame < loc->end()) {
 
-		if (loc && !_have_captured) {
-				
-			/* first capture.
-			   
-			   note: later captures that extend the session length get
-			   handled because of playlist length changes.
-			*/
+			/* stopped recording before current end */
+
+			if (_end_location_is_free) {
+
+				/* first capture for this session, move end back to where we are */
+
+				change_end = true;
+			} 
+
+		} else if (_transport_frame > loc->end()) {
 			
-			add_undo (rettype<void>(bind (slot (*loc, &Location::set_end), loc->end())));
-			add_redo (rettype<void>(bind (slot (*loc, &Location::set_end), _transport_frame)));
+			/* stopped recording after the current end, extend it */
+
+			change_end = true;
+		}
+		
+		if (change_end) {
+                        XMLNode &before = loc->get_state();
+                        loc->set_end(_transport_frame);
+                        XMLNode &after = loc->get_state();
+                        add_command (new MementoCommand<Location>(*loc, &before, &after));
 		}
 
+		_end_location_is_free = false;
 		_have_captured = true;
 	}
 
-	for (DiskStreamList::iterator i = diskstreams.begin(); i != diskstreams.end(); ++i) {
+	for (DiskstreamList::iterator i = dsl->begin(); i != dsl->end(); ++i) {
 		(*i)->transport_stopped (*now, xnow, abort);
 	}
 	
-	for (RouteList::iterator i = routes.begin(); i != routes.end(); ++i) {
+	boost::shared_ptr<RouteList> r = routes.reader ();
+
+	for (RouteList::iterator i = r->begin(); i != r->end(); ++i) {
 		if (!(*i)->hidden()) {
 			(*i)->set_pending_declick (0);
 		}
 	}
-
+	
 	if (did_record) {
 		commit_reversible_command ();
 	}	
-
+	
 	if (_engine.running()) {
 		update_latency_compensation (true, abort);
 	}
 
-	if (auto_return || (post_transport_work & PostTransportLocate) || synced_to_jack()) {
-
+	if ((Config->get_slave_source() == None && Config->get_auto_return()) || (post_transport_work & PostTransportLocate) || synced_to_jack()) {
+		
 		if (pending_locate_flush) {
 			flush_all_redirects ();
 		}
-
-		if ((auto_return || synced_to_jack()) && !(post_transport_work & PostTransportLocate)) {
+		
+		if (((Config->get_slave_source() == None && Config->get_auto_return()) || synced_to_jack()) && !(post_transport_work & PostTransportLocate)) {
 
 			_transport_frame = last_stop_frame;
 
-			if (synced_to_jack()) {
+			if (synced_to_jack() && !play_loop) {
+				// cerr << "non-realtimestop: transport locate to " << _transport_frame << endl;
 				_engine.transport_locate (_transport_frame);
 			}
 		} 
@@ -362,46 +391,61 @@ Session::non_realtime_stop (bool abort)
 	}
 #endif
 
-		for (DiskStreamList::iterator i = diskstreams.begin(); i != diskstreams.end(); ++i) {
+		for (DiskstreamList::iterator i = dsl->begin(); i != dsl->end(); ++i) {
 			if (!(*i)->hidden()) {
 				if ((*i)->speed() != 1.0f || (*i)->speed() != -1.0f) {
-					(*i)->seek ((jack_nframes_t) (_transport_frame * (double) (*i)->speed()));
+					(*i)->seek ((nframes_t) (_transport_frame * (double) (*i)->speed()));
 				}
 				else {
 					(*i)->seek (_transport_frame);
 				}
 			}
+			if (on_entry != g_atomic_int_get (&butler_should_do_transport_work)) {
+				finished = false;
+				/* we will be back */
+				return;
+			}
 		}
-
-		deliver_mmc (MIDI::MachineControl::cmdLocate, _transport_frame);
-
 #ifdef LEAVE_TRANSPORT_UNADJUSTED
 	}
 #endif
 
-       last_stop_frame = _transport_frame;
+	last_stop_frame = _transport_frame;
 
 	send_full_time_code ();
 	deliver_mmc (MIDI::MachineControl::cmdStop, 0);
 	deliver_mmc (MIDI::MachineControl::cmdLocate, _transport_frame);
 
 	if (did_record) {
-		atomic_set (&_record_status, Disabled);
-		RecordDisabled (); /* EMIT SIGNAL */
+
+		/* XXX its a little odd that we're doing this here
+		   when realtime_stop(), which has already executed,
+		   will have done this.
+		   JLC - so let's not because it seems unnecessary and breaks loop record
+		*/
+#if 0
+		if (!Config->get_latched_record_enable()) {
+			g_atomic_int_set (&_record_status, Disabled);
+		} else {
+			g_atomic_int_set (&_record_status, Enabled);
+		}
+		RecordStateChanged (); /* emit signal */
+#endif
 	}
 	
 	if ((post_transport_work & PostTransportLocate) && get_record_enabled()) {
 		/* capture start has been changed, so save pending state */
 		save_state ("", true);
+		saved = true;
 	}
 
-	/* always try to get rid of this */
+        /* always try to get rid of this */
 
-	remove_pending_capture_state ();
-
-	/* save the current state of things if appropriate */
+        remove_pending_capture_state ();
 	
-	if (did_record) {
+	/* save the current state of things if appropriate */
+
+	if (did_record && !saved) {
 		save_state (_current_snapshot_name);
 	}
 
@@ -413,7 +457,7 @@ Session::non_realtime_stop (bool abort)
 		_play_range = false;
 
 		/* do not turn off autoloop on stop */
-
+		
 	}
 
 	PositionChanged (_transport_frame); /* EMIT SIGNAL */
@@ -421,7 +465,7 @@ Session::non_realtime_stop (bool abort)
 
 	/* and start it up again if relevant */
 
-	if ((post_transport_work & PostTransportLocate) && _slave_type == None && pending_locate_roll) {
+	if ((post_transport_work & PostTransportLocate) && Config->get_slave_source() == None && pending_locate_roll) {
 		request_transport_speed (1.0);
 		pending_locate_roll = false;
 	}
@@ -452,7 +496,7 @@ Session::check_declick_out ()
 }
 
 void
-Session::set_auto_loop (bool yn)
+Session::set_play_loop (bool yn)
 {
 	/* Called from event-handling context */
 	
@@ -462,7 +506,7 @@ Session::set_auto_loop (bool yn)
 	
 	set_dirty();
 
-	if (yn && seamless_loop && synced_to_jack()) {
+	if (yn && Config->get_seamless_loop() && synced_to_jack()) {
 		warning << _("Seamless looping cannot be supported while Ardour is using JACK transport.\n"
 			     "Recommend changing the configured options")
 			<< endmsg;
@@ -470,16 +514,17 @@ Session::set_auto_loop (bool yn)
 	}
 
 	
-	if ((auto_loop = yn)) {
+	if ((play_loop = yn)) {
 
 		Location *loc;
 
 		
 		if ((loc = _locations.auto_loop_location()) != 0) {
 
-			if (seamless_loop) {
+			if (Config->get_seamless_loop()) {
 				// set all diskstreams to use internal looping
-				for (DiskStreamList::iterator i = diskstreams.begin(); i != diskstreams.end(); ++i) {
+				boost::shared_ptr<DiskstreamList> dsl = diskstreams.reader();
+				for (DiskstreamList::iterator i = dsl->begin(); i != dsl->end(); ++i) {
 					if (!(*i)->hidden()) {
 						(*i)->set_loop (loc);
 					}
@@ -487,7 +532,8 @@ Session::set_auto_loop (bool yn)
 			}
 			else {
 				// set all diskstreams to NOT use internal looping
-				for (DiskStreamList::iterator i = diskstreams.begin(); i != diskstreams.end(); ++i) {
+				boost::shared_ptr<DiskstreamList> dsl = diskstreams.reader();
+				for (DiskstreamList::iterator i = dsl->begin(); i != dsl->end(); ++i) {
 					if (!(*i)->hidden()) {
 						(*i)->set_loop (0);
 					}
@@ -517,32 +563,33 @@ Session::set_auto_loop (bool yn)
 		clear_events (Event::AutoLoop);
 
 		// set all diskstreams to NOT use internal looping
-		for (DiskStreamList::iterator i = diskstreams.begin(); i != diskstreams.end(); ++i) {
+		boost::shared_ptr<DiskstreamList> dsl = diskstreams.reader();
+		for (DiskstreamList::iterator i = dsl->begin(); i != dsl->end(); ++i) {
 			if (!(*i)->hidden()) {
 				(*i)->set_loop (0);
 			}
 		}
 		
 	}
-	
-	ControlChanged (AutoLoop); /* EMIT SIGNAL */
 }
 
 void
 Session::flush_all_redirects ()
 {
-	for (RouteList::iterator i = routes.begin(); i != routes.end(); ++i) {
+	boost::shared_ptr<RouteList> r = routes.reader ();
+
+	for (RouteList::iterator i = r->begin(); i != r->end(); ++i) {
 		(*i)->flush_redirects ();
 	}
 }
 
 void
-Session::start_locate (jack_nframes_t target_frame, bool with_roll, bool with_flush, bool with_loop)
+Session::start_locate (nframes_t target_frame, bool with_roll, bool with_flush, bool with_loop)
 {
 	if (synced_to_jack()) {
 
 		float sp;
-		jack_nframes_t pos;
+		nframes_t pos;
 
 		_slave->speed_and_position (sp, pos);
 
@@ -567,9 +614,9 @@ Session::start_locate (jack_nframes_t target_frame, bool with_roll, bool with_fl
 }
 
 void
-Session::locate (jack_nframes_t target_frame, bool with_roll, bool with_flush, bool with_loop)
+Session::locate (nframes_t target_frame, bool with_roll, bool with_flush, bool with_loop)
 {
-	if (actively_recording()) {
+	if (actively_recording() && !with_loop) {
 		return;
 	}
 
@@ -595,7 +642,7 @@ Session::locate (jack_nframes_t target_frame, bool with_roll, bool with_flush, b
 		} 
 	}
 
-	if (transport_rolling() && !auto_play && !with_roll && !(synced_to_jack() && auto_loop)) {
+	if (transport_rolling() && (!auto_play_legal || !Config->get_auto_play()) && !with_roll && !(synced_to_jack() && play_loop)) {
 		realtime_stop (false);
 	} 
 
@@ -613,7 +660,7 @@ Session::locate (jack_nframes_t target_frame, bool with_roll, bool with_flush, b
 
 		/* this is functionally what clear_clicks() does but with a tentative lock */
 
-		TentativeRWLockMonitor clickm (click_lock, true, __LINE__, __FILE__);
+		Glib::RWLock::WriterLock clickm (click_lock, Glib::TRY_LOCK);
 	
 		if (clickm.locked()) {
 			
@@ -627,28 +674,23 @@ Session::locate (jack_nframes_t target_frame, bool with_roll, bool with_flush, b
 
 	if (with_roll) {
 		/* switch from input if we're going to roll */
-		if (Config->get_use_hardware_monitoring()) {
-			/* Even though this is called from RT context we are using
-			   a non-tentative rwlock here,  because the action must occur.
-			   The rarity and short potential lock duration makes this "OK"
-			*/
-			RWLockMonitor dsm (diskstream_lock, false, __LINE__, __FILE__);
-			for (DiskStreamList::iterator i = diskstreams.begin(); i != diskstreams.end(); ++i) {
+		if (Config->get_monitoring_model() == HardwareMonitoring) {
+
+			boost::shared_ptr<DiskstreamList> dsl = diskstreams.reader();
+
+			for (DiskstreamList::iterator i = dsl->begin(); i != dsl->end(); ++i) {
 				if ((*i)->record_enabled ()) {
 					//cerr << "switching from input" << __FILE__ << __LINE__ << endl << endl;
-					(*i)->monitor_input (!auto_input);
+					(*i)->monitor_input (!Config->get_auto_input());
 				}
 			}
 		}
 	} else {
 		/* otherwise we're going to stop, so do the opposite */
-		if (Config->get_use_hardware_monitoring()) {
-			/* Even though this is called from RT context we are using
-			   a non-tentative rwlock here,  because the action must occur.
-			   The rarity and short potential lock duration makes this "OK"
-			*/
-			RWLockMonitor dsm (diskstream_lock, false, __LINE__, __FILE__);
-			for (DiskStreamList::iterator i = diskstreams.begin(); i != diskstreams.end(); ++i) {
+		if (Config->get_monitoring_model() == HardwareMonitoring) {
+			boost::shared_ptr<DiskstreamList> dsl = diskstreams.reader();
+
+			for (DiskstreamList::iterator i = dsl->begin(); i != dsl->end(); ++i) {
 				if ((*i)->record_enabled ()) {
 					//cerr << "switching to input" << __FILE__ << __LINE__ << endl << endl;
 					(*i)->monitor_input (true);
@@ -657,13 +699,29 @@ Session::locate (jack_nframes_t target_frame, bool with_roll, bool with_flush, b
 		}
 	}
 
-	/* cancel autoloop if transport pos outside of loop range */
-	if (auto_loop) {
+	/* cancel looped playback if transport pos outside of loop range */
+	if (play_loop) {
 		Location* al = _locations.auto_loop_location();
 		
 		if (al && (_transport_frame < al->start() || _transport_frame > al->end())) {
 			// cancel looping directly, this is called from event handling context
-			set_auto_loop(false);
+			set_play_loop (false);
+		}
+		else if (al && _transport_frame == al->start()) {
+			if (with_loop) {
+				// this is only necessary for seamless looping
+
+				boost::shared_ptr<DiskstreamList> dsl = diskstreams.reader();
+				
+				for (DiskstreamList::iterator i = dsl->begin(); i != dsl->end(); ++i) {
+					if ((*i)->record_enabled ()) {
+						// tell it we've looped, so it can deal with the record state
+						(*i)->transport_looped(_transport_frame);
+					}
+				}
+			}
+
+			TransportLooped(); // EMIT SIGNAL
 		}
 	}
 	
@@ -685,14 +743,11 @@ Session::set_transport_speed (float speed, bool abort)
 
 	if (transport_rolling() && speed == 0.0) {
 
-		if (Config->get_use_hardware_monitoring())
+		if (Config->get_monitoring_model() == HardwareMonitoring)
 		{
-			/* Even though this is called from RT context we are using
-			   a non-tentative rwlock here,  because the action must occur.
-			   The rarity and short potential lock duration makes this "OK"
-			*/
-			RWLockMonitor dsm (diskstream_lock, false, __LINE__, __FILE__);
-			for (DiskStreamList::iterator i = diskstreams.begin(); i != diskstreams.end(); ++i) {
+			boost::shared_ptr<DiskstreamList> dsl = diskstreams.reader();
+
+			for (DiskstreamList::iterator i = dsl->begin(); i != dsl->end(); ++i) {
 				if ((*i)->record_enabled ()) {
 					//cerr << "switching to input" << __FILE__ << __LINE__ << endl << endl;
 					(*i)->monitor_input (true);	
@@ -705,17 +760,19 @@ Session::set_transport_speed (float speed, bool abort)
 		} else {
 			stop_transport (abort);
 		}
-
+		
 	} else if (transport_stopped() && speed == 1.0) {
 
-		if (Config->get_use_hardware_monitoring()) {
-			/* Even though this is called from RT context we are using
-			   a non-tentative rwlock here,  because the action must occur.
-			   The rarity and short potential lock duration makes this "OK"
-			*/
-			RWLockMonitor dsm (diskstream_lock, false, __LINE__, __FILE__);
-			for (DiskStreamList::iterator i = diskstreams.begin(); i != diskstreams.end(); ++i) {
-				if (auto_input && (*i)->record_enabled ()) {
+		if (!get_record_enabled() && Config->get_stop_at_session_end() && _transport_frame >= current_end_frame()) {
+			return;
+		}
+
+		if (Config->get_monitoring_model() == HardwareMonitoring) {
+
+			boost::shared_ptr<DiskstreamList> dsl = diskstreams.reader();
+
+			for (DiskstreamList::iterator i = dsl->begin(); i != dsl->end(); ++i) {
+				if (Config->get_auto_input() && (*i)->record_enabled ()) {
 					//cerr << "switching from input" << __FILE__ << __LINE__ << endl << endl;
 					(*i)->monitor_input (false);	
 				}
@@ -730,6 +787,10 @@ Session::set_transport_speed (float speed, bool abort)
 
 	} else {
 
+		if (!get_record_enabled() && Config->get_stop_at_session_end() && _transport_frame >= current_end_frame()) {
+			return;
+		}
+
 		if ((synced_to_jack()) && speed != 0.0 && speed != 1.0) {
 			warning << _("Global varispeed cannot be supported while Ardour is connected to JACK transport control")
 				<< endmsg;
@@ -754,14 +815,15 @@ Session::set_transport_speed (float speed, bool abort)
 		   before the last stop, then we have to do extra work.
 		*/
 
-		if ((_transport_speed && speed * _transport_speed < 0.0f) || (_last_transport_speed * speed < 0.0f)) {
+		if ((_transport_speed && speed * _transport_speed < 0.0f) || (_last_transport_speed * speed < 0.0f) || (_last_transport_speed == 0.0f && speed < 0.0f)) {
 			post_transport_work = PostTransportWork (post_transport_work | PostTransportReverse);
 		}
 		
 		_last_transport_speed = _transport_speed;
 		_transport_speed = speed;
 		
-		for (DiskStreamList::iterator i = diskstreams.begin(); i != diskstreams.end(); ++i) {
+		boost::shared_ptr<DiskstreamList> dsl = diskstreams.reader();
+		for (DiskstreamList::iterator i = dsl->begin(); i != dsl->end(); ++i) {
 			if ((*i)->realtime_set_speed ((*i)->speed(), true)) {
 				post_transport_work = PostTransportWork (post_transport_work | PostTransportSpeed);
 			}
@@ -779,7 +841,7 @@ Session::stop_transport (bool abort)
 	if (_transport_speed == 0.0f) {
 		return;
 	}
-
+	
 	if (actively_recording() && !(transport_sub_state & StopPendingCapture) && 
 	    _worst_output_latency > current_block_size) 
 	{
@@ -802,9 +864,11 @@ Session::stop_transport (bool abort)
 		return;
 	} 
 
+
 	if ((transport_sub_state & PendingDeclickOut) == 0) {
 		transport_sub_state |= PendingDeclickOut;
 		/* we'll be called again after the declick */
+		pending_abort = abort;
 		return;
 	}
 
@@ -823,13 +887,15 @@ Session::start_transport ()
 
 	switch (record_status()) {
 	case Enabled:
-		if (!punch_in) {
+		if (!Config->get_punch_in()) {
 			enable_record ();
 		}
 		break;
 
 	case Recording:
-		disable_record (false);
+		if (!play_loop) {
+			disable_record (false);
+		}
 		break;
 
 	default:
@@ -851,7 +917,8 @@ Session::actually_start_transport ()
 	transport_sub_state |= PendingDeclickIn;
 	_transport_speed = 1.0;
 	
-	for (DiskStreamList::iterator i = diskstreams.begin(); i != diskstreams.end(); ++i) {
+	boost::shared_ptr<DiskstreamList> dsl = diskstreams.reader();
+	for (DiskstreamList::iterator i = dsl->begin(); i != dsl->end(); ++i) {
 		(*i)->realtime_set_speed ((*i)->speed(), true);
 	}
 
@@ -878,8 +945,7 @@ Session::post_transport ()
 
 	if (post_transport_work & PostTransportLocate) {
 
-		if ((auto_play && !_exporting) || (post_transport_work & PostTransportRoll)) {
-			
+		if (((Config->get_slave_source() == None && (auto_play_legal && Config->get_auto_play())) && !_exporting) || (post_transport_work & PostTransportRoll)) {
 			start_transport ();
 			
 		} else {
@@ -893,15 +959,7 @@ Session::post_transport ()
 }
 
 void
-Session::set_rf_speed (float speed)
-{
-	rf_speed = speed;
-	cumulative_rf_motion = 0;
-	reset_rf_scale (0);
-}
-
-void
-Session::reset_rf_scale (jack_nframes_t motion)
+Session::reset_rf_scale (nframes_t motion)
 {
 	cumulative_rf_motion += motion;
 
@@ -920,31 +978,24 @@ Session::reset_rf_scale (jack_nframes_t motion)
 	}
 }
 
-int
-Session::set_slave_source (SlaveSource src, jack_nframes_t frame)
+void
+Session::set_slave_source (SlaveSource src)
 {
 	bool reverse = false;
 	bool non_rt_required = false;
 
 	if (_transport_speed) {
 		error << _("please stop the transport before adjusting slave settings") << endmsg;
-		/* help out non-MVC friendly UI's by telling them the slave type changed */
-		 ControlChanged (SlaveType); /* EMIT SIGNAL */
-		return 0;
+		return;
 	}
 
-	if (src == _slave_type) {
-		return 0;
-	} 
-
 // 	if (src == JACK && Config->get_jack_time_master()) {
-// 		return -1;
+// 		return;
 // 	}
 	
 	if (_slave) {
 		delete _slave;
 		_slave = 0;
-		_slave_type = None;
 	}
 
 	if (_transport_speed < 0.0) {
@@ -955,7 +1006,7 @@ Session::set_slave_source (SlaveSource src, jack_nframes_t frame)
 	case None:
 		stop_transport ();
 		break;
-
+		
 	case MTC:
 		if (_mtc_port) {
 			try {
@@ -963,12 +1014,12 @@ Session::set_slave_source (SlaveSource src, jack_nframes_t frame)
 			}
 
 			catch (failed_constructor& err) {
-				return -1;
+				return;
 			}
 
 		} else {
 			error << _("No MTC port defined: MTC slaving is impossible.") << endmsg;
-			return -1;
+			return;
 		}
 		_desired_transport_speed = _transport_speed;
 		break;
@@ -978,10 +1029,11 @@ Session::set_slave_source (SlaveSource src, jack_nframes_t frame)
 		_desired_transport_speed = _transport_speed;
 		break;
 	};
-	
-	_slave_type = src;
 
-	for (DiskStreamList::iterator i = diskstreams.begin(); i != diskstreams.end(); ++i) {
+	Config->set_slave_source (src);
+	
+	boost::shared_ptr<DiskstreamList> dsl = diskstreams.reader();
+	for (DiskstreamList::iterator i = dsl->begin(); i != dsl->end(); ++i) {
 		if (!(*i)->hidden()) {
 			if ((*i)->realtime_set_speed ((*i)->speed(), true)) {
 				non_rt_required = true;
@@ -1000,9 +1052,6 @@ Session::set_slave_source (SlaveSource src, jack_nframes_t frame)
 	}
 
 	set_dirty();
-	ControlChanged (SlaveType); /* EMIT SIGNAL */
-	
-	return 0;
 }
 
 void
@@ -1013,7 +1062,7 @@ Session::reverse_diskstream_buffers ()
 }
 
 void
-Session::set_diskstream_speed (DiskStream* stream, float speed)
+Session::set_diskstream_speed (Diskstream* stream, float speed)
 {
 	if (stream->realtime_set_speed (speed, false)) {
 		post_transport_work = PostTransportWork (post_transport_work | PostTransportSpeed);
@@ -1051,8 +1100,6 @@ Session::set_play_range (bool yn)
 			Event* ev = new Event (Event::SetTransportSpeed, Event::Add, Event::Immediate, 0, 0.0f, false);
 			merge_event (ev);
 		}
-
- 		 ControlChanged (PlayRange); /* EMIT SIGNAL */
 	}
 }
 
@@ -1085,7 +1132,7 @@ Session::setup_auto_play ()
 			/* locating/stopping is subject to delays for declicking.
 			 */
 			
-			jack_nframes_t requested_frame = (*i).end;
+			nframes_t requested_frame = (*i).end;
 			
 			if (requested_frame > current_block_size) {
 				requested_frame -= current_block_size;
@@ -1118,10 +1165,10 @@ Session::setup_auto_play ()
 }
 
 void
-Session::request_bounded_roll (jack_nframes_t start, jack_nframes_t end)
+Session::request_bounded_roll (nframes_t start, nframes_t end)
 {
 	request_stop ();
-	Event *ev = new Event (Event::StopOnce, Event::Replace, Event::Immediate, end, 0.0);
+ 	Event *ev = new Event (Event::StopOnce, Event::Replace, end, Event::Immediate, 0.0);
 	queue_event (ev);
 	request_locate (start, true);
 }
@@ -1129,6 +1176,8 @@ Session::request_bounded_roll (jack_nframes_t start, jack_nframes_t end)
 void
 Session::engine_halted ()
 {
+	bool ignored;
+
 	/* there will be no more calls to process(), so
 	   we'd better clean up for ourselves, right now.
 
@@ -1136,12 +1185,12 @@ Session::engine_halted ()
 	   the picture.
 	*/
 
-	atomic_set (&butler_should_do_transport_work, 0);
+	g_atomic_int_set (&butler_should_do_transport_work, 0);
 	post_transport_work = PostTransportWork (0);
 	stop_butler ();
 	
 	realtime_stop (false);
-	non_realtime_stop (false);
+	non_realtime_stop (false, 0, ignored);
 	transport_sub_state = 0;
 
 	TransportStateChange (); /* EMIT SIGNAL */
@@ -1172,18 +1221,18 @@ Session::update_latency_compensation (bool with_stop, bool abort)
 		return;
 	}
 
-	RWLockMonitor lm (route_lock, false, __LINE__, __FILE__);
-	RWLockMonitor lm2 (diskstream_lock, false, __LINE__, __FILE__);
 	_worst_track_latency = 0;
 
-	for (RouteList::iterator i = routes.begin(); i != routes.end(); ++i) {
+	boost::shared_ptr<RouteList> r = routes.reader ();
+
+	for (RouteList::iterator i = r->begin(); i != r->end(); ++i) {
 		if (with_stop) {
-			(*i)->transport_stopped (abort, (post_transport_work & PostTransportLocate), 
-						 (!(post_transport_work & PostTransportLocate) || pending_locate_flush));
+			(*i)->handle_transport_stopped (abort, (post_transport_work & PostTransportLocate), 
+							(!(post_transport_work & PostTransportLocate) || pending_locate_flush));
 		}
 
-		jack_nframes_t old_latency = (*i)->signal_latency ();
-		jack_nframes_t track_latency = (*i)->update_total_latency ();
+		nframes_t old_latency = (*i)->signal_latency ();
+		nframes_t track_latency = (*i)->update_total_latency ();
 
 		if (old_latency != track_latency) {
 			update_jack = true;
@@ -1194,7 +1243,7 @@ Session::update_latency_compensation (bool with_stop, bool abort)
 		}
 	}
 
-	for (RouteList::iterator i = routes.begin(); i != routes.end(); ++i) {
+	for (RouteList::iterator i = r->begin(); i != r->end(); ++i) {
 		(*i)->set_latency_delay (_worst_track_latency);
 	}
 
@@ -1204,12 +1253,14 @@ Session::update_latency_compensation (bool with_stop, bool abort)
 		_engine.update_total_latencies ();
 	}
 
-	set_worst_io_latencies (false);
+	set_worst_io_latencies ();
 
 	/* reflect any changes in latencies into capture offsets
 	*/
 	
-	for (DiskStreamList::iterator i = diskstreams.begin(); i != diskstreams.end(); ++i) {
+	boost::shared_ptr<DiskstreamList> dsl = diskstreams.reader();
+
+	for (DiskstreamList::iterator i = dsl->begin(); i != dsl->end(); ++i) {
 		(*i)->set_capture_offset ();
 	}
 }
@@ -1219,3 +1270,19 @@ Session::update_latency_compensation_proxy (void* ignored)
 {
 	update_latency_compensation (false, false);
 }
+
+void
+Session::allow_auto_play (bool yn)
+{
+	auto_play_legal = yn;
+}
+
+void
+Session::reset_jack_connection (jack_client_t* jack)
+{
+	JACK_Slave* js;
+
+	if (_slave && ((js = dynamic_cast<JACK_Slave*> (_slave)) != 0)) {
+		js->reset_client (jack);
+	}
+}
