@@ -15,7 +15,6 @@
     along with this program; if not, write to the Free Software
     Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
 
-    $Id: vst_plugin.cc,v 1.20 2006/02/11 14:46:05 sampo Exp $
 */
 
 #include <algorithm>
@@ -41,8 +40,6 @@
 
 #include <vst/aeffectx.h>
 
-#include <midi++/manager.h>
-
 #include <ardour/ardour.h>
 #include <ardour/session.h>
 #include <ardour/audioengine.h>
@@ -54,6 +51,7 @@
 #include <locale.h>
 
 using namespace ARDOUR;
+using namespace PBD;
 using std::min;
 using std::max;
 
@@ -80,7 +78,7 @@ VSTPlugin::VSTPlugin (AudioEngine& e, Session& session, FSTHandle* h)
 
 	_plugin->dispatcher (_plugin, effSetProgram, 0, 0, NULL, 0.0f);
 	
-	Plugin::setup_midi_controls ();
+	Plugin::setup_controls ();
 }
 
 VSTPlugin::VSTPlugin (const VSTPlugin &other)
@@ -93,34 +91,24 @@ VSTPlugin::VSTPlugin (const VSTPlugin &other)
 	}
 	_plugin = _fst->plugin;
 
-	Plugin::setup_midi_controls ();
+	Plugin::setup_controls ();
 }
 
 VSTPlugin::~VSTPlugin ()
 {
 	deactivate ();
-	GoingAway (this); /* EMIT SIGNAL */
+	GoingAway (); /* EMIT SIGNAL */
 	fst_close (_fst);
 }
 
 void
-VSTPlugin::set_block_size (jack_nframes_t nframes)
+VSTPlugin::set_block_size (nframes_t nframes)
 {
 	deactivate ();
 	_plugin->dispatcher (_plugin, effSetBlockSize, 0, nframes, NULL, 0.0f);
 	activate ();
 }
 
-void
-VSTPlugin::store_state (PluginState& state)
-{
-}
-
-void
-VSTPlugin::restore_state (PluginState& state)
-{
-}
-
 float
 VSTPlugin::default_value (uint32_t port)
 {
@@ -132,13 +120,6 @@ VSTPlugin::set_parameter (uint32_t which, float val)
 {
 	_plugin->setParameter (_plugin, which, val);
 	ParameterChanged (which, val); /* EMIT SIGNAL */
-
-	if (session().get_midi_feedback()) {
-		
-		if (which < parameter_count() && midi_controls[which]) {
-			midi_controls[which]->send_feedback (val);
-		}
-	}
 }
 
 float
@@ -177,28 +158,27 @@ VSTPlugin::get_state()
 		string path;
 		struct stat sbuf;
 
-		path = getenv ("HOME");
-		path += "/.ardour/vst";
+		path = get_user_ardour_path ();
+		path += "vst";
 
 		if (stat (path.c_str(), &sbuf)) {
 			if (errno == ENOENT) {
-				if (mkdir (path.c_str(), 0600)) {
-					error << compose (_("cannot create VST chunk directory: %1"),
-							  strerror (errno))
+				if (g_mkdir_with_parents (path.c_str(), 0600)) {
+					error << string_compose (_("cannot create VST chunk directory: %1"),
+								 strerror (errno))
 					      << endmsg;
 					return *root;
 				}
 
 			} else {
 
-				error << compose (_("cannot check VST chunk directory: %1"),
-						  strerror (errno))
-				      << endmsg;
+				warning << string_compose (_("cannot check VST chunk directory: %1"), strerror (errno))
+					<< endmsg;
 				return *root;
 			}
 
 		} else if (!S_ISDIR (sbuf.st_mode)) {
-			error << compose (_("%1 exists but is not a directory"), path)
+			error << string_compose (_("%1 exists but is not a directory"), path)
 			      << endmsg;
 			return *root;
 		}
@@ -220,7 +200,7 @@ VSTPlugin::get_state()
 			char index[64];
 			char val[32];
 			snprintf (index, sizeof (index), "param_%ld", n);
-			snprintf (val, sizeof (val), "%f", _plugin->getParameter (_plugin, n));
+			snprintf (val, sizeof (val), "%.12g", _plugin->getParameter (_plugin, n));
 			parameters->add_property (index, val);
 		}
 
@@ -253,6 +233,7 @@ VSTPlugin::set_state(const XMLNode& node)
 		for (i = child->properties().begin(); i != child->properties().end(); ++i) {
 			long param;
 			float val;
+
 			sscanf ((*i)->name().c_str(), "param_%ld", &param);
 			sscanf ((*i)->value().c_str(), "%f", &val);
 
@@ -365,7 +346,7 @@ VSTPlugin::describe_parameter (uint32_t param)
 	return name;
 }
 
-jack_nframes_t
+nframes_t
 VSTPlugin::latency () const
 {
 	return _plugin->initialDelay;
@@ -384,7 +365,7 @@ VSTPlugin::automatable () const
 }
 
 int
-VSTPlugin::connect_and_run (vector<Sample*>& bufs, uint32_t maxbuf, int32_t& in_index, int32_t& out_index, jack_nframes_t nframes, jack_nframes_t offset)
+VSTPlugin::connect_and_run (vector<Sample*>& bufs, uint32_t maxbuf, int32_t& in_index, int32_t& out_index, nframes_t nframes, nframes_t offset)
 {
 	float *ins[_plugin->numInputs];
 	float *outs[_plugin->numOutputs];
@@ -487,3 +468,33 @@ VSTPlugin::print_parameter (uint32_t param, char *buf, uint32_t len) const
 
 	memmove (buf, first_nonws, strlen (buf) - (first_nonws - buf) + 1);
 }
+
+PluginPtr
+VSTPluginInfo::load (Session& session)
+{
+	try {
+		PluginPtr plugin;
+
+		if (Config->get_use_vst()) {
+			FSTHandle* handle;
+			
+			handle = fst_load(path.c_str());
+	
+			if ( (int)handle == -1) {
+				error << string_compose(_("VST: cannot load module from \"%1\""), path) << endmsg;
+			} else {
+				plugin.reset (new VSTPlugin (session.engine(), session, handle));
+			}
+		} else {
+			error << _("You asked ardour to not use any VST plugins") << endmsg;
+			return PluginPtr ((Plugin*) 0);
+		}
+
+		plugin->set_info(PluginInfoPtr(new VSTPluginInfo(*this)));
+		return plugin;
+	}	
+
+	catch (failed_constructor &err) {
+		return PluginPtr ((Plugin*) 0);
+	}
+}
